# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import math
import torch
import torch.nn as nn
import torch.nn.functional as F
import bmtrain as bmt

class RelativePositionEmbedding(bmt.DistributedModule):
    """ Relative Position Embedding <https://arxiv.org/abs/1803.02155>

    Args:
        num_heads (int): number of heads used in attention module.
        num_buckets (int, optional): Defaults to 32.
        max_distance (int, optional): Defaults to 128.
        bidirectional (bool, optional): Defaults to False.
        dtype (optional): Defaults to torch.half.
        init_mean (float, optional): Defaults to 0.0.
        init_std (float, optional): Defaults to 1.
    """

    def __init__(self, num_heads : int, 
                       num_buckets : int = 32, 
                       max_distance : int = 128, 
                       bidirectional : bool = False, 
                       dtype = torch.half,
                       init_mean : float = 0.0,
                       init_std : float = 1,
                    ):

        super().__init__()

        self.relative_attention_bias = bmt.DistributedParameter(
            torch.empty(num_buckets, num_heads, dtype = dtype), 
            init_method = bmt.ParameterInitializer(nn.init.normal_, mean = init_mean, std = init_std)
        )
        self.num_heads = num_heads
        self.num_buckets = num_buckets
        self.max_distance = max_distance
        self.bidirectional = bidirectional

    def forward(self, query, key):
        """ Provides relative position embeddings for key and query of `num_heads` attention heads. 

        Args:
            query (:obj:`int`): Length of query or query tensor.  
            key (:obj:`int`): Length of key or key tenser.
        Return:
            :obj:`torch.Tensor` of shape ``(num_heads, query_len, key_len)``: Relative position embedding.
        """

        part_buckets = self.num_buckets // (2 if self.bidirectional else 1)
        exact_buckets = part_buckets // 2
        log_buckets = part_buckets - exact_buckets

        if isinstance(query, int):
            query = torch.arange(query, dtype=torch.long, device="cuda")
        if isinstance(key, int):
            key = torch.arange(key, dtype=torch.long, device="cuda")

        if query.dim() == 1:
            relative_position = query[:, None] - key[None, :]
        else:
            relative_position = query[:, :, None] - key[:, None, :]

        neg_pos = relative_position < 0
        relative_position = relative_position.abs()

        small_pos = relative_position < exact_buckets

        log_pos = (torch.clamp(
            torch.log(relative_position.float() / exact_buckets) / math.log(self.max_distance / exact_buckets),
            0,
            0.9999
        ) * log_buckets).long() + exact_buckets

        buckets = torch.where(small_pos, relative_position, log_pos)
        if self.bidirectional:
            buckets = torch.where(
                neg_pos,
                buckets + part_buckets,
                buckets
            )
        else:
            buckets = torch.masked_fill(
                buckets,
                neg_pos,
                0,
            )
        if query.dim() == 1:
            return F.embedding(buckets, self.relative_attention_bias, padding_idx = -1).permute(2, 0, 1).contiguous()
        else:
            return F.embedding(buckets, self.relative_attention_bias, padding_idx = -1).permute(-1, -3, -2).contiguous()


class RotaryEmbedding(nn.Module):
    """`Rotary Position Embedding <https://arxiv.org/abs/2104.09864v2>

    Args:
        rotary_dim (int): rotary dimension
    """
    def __init__(self, rotary_dim: int):
        # Implementation reference https://github.com/huggingface/transformers/blob/master/src/transformers/models/gptj/modeling_gptj.py
        super().__init__()
        self.rotary_dim = rotary_dim

    def fixed_pos_embedding(self, x, seq_len=None, dtype = torch.float):
        dim = x.shape[-1]
        inv_freq = 1.0 / (10000 ** (torch.arange(0, dim, 2) / dim))
        sinusoid_inp = torch.einsum("i , j -> i j", torch.arange(seq_len), inv_freq).to(x.device).to(dtype)
        return torch.sin(sinusoid_inp), torch.cos(sinusoid_inp)

    def rotate_every_two(self, x):
        if x.dim() == 4:
            x1 = x[:, :, :, ::2]
            x2 = x[:, :, :, 1::2]
        else:
            x1 = x[:, :, ::2]
            x2 = x[:, :, 1::2]
        x = torch.stack((-x2, x1), axis=-1)
        return x.flatten(-2)  # in einsum notation: rearrange(x, '... d j -> ... (d j)')

    def apply_rotary_pos_emb(self, x, sincos, offset=0):
        sin, cos = map(lambda t: t[None, offset : x.shape[-2] + offset, :].repeat_interleave(2, 2), sincos)
        # einsum notation for lambda t: repeat(t[offset:x.shape[1]+offset,:], "n d -> () n () (d j)", j=2)
        return (x * cos) + (self.rotate_every_two(x) * sin)

    def forward(self, h_q, h_k):
        """
        Args:
            h_q : (batch_size, num_head, len_q, dim_head)
            h_k : (batch_size, k_num_head, len_k, dim_head)

        Return:
            h_q : (batch_size, num_head, len_q, dim_head)
            h_k : (batch_size, k_num_head, len_k, dim_head)
        """
        if h_q.dim() == 4:
            q_rot = h_q[:, :, :, :self.rotary_dim]
            q_pass = h_q[:, :, :, self.rotary_dim:]
            k_rot = h_k[:, :, :, :self.rotary_dim]
            k_pass = h_k[:, :, :, self.rotary_dim:]
        else:
            q_rot = h_q[:, :, :self.rotary_dim]
            q_pass = h_q[:, :, self.rotary_dim:]
            k_rot = h_k[:, :, :self.rotary_dim]
            k_pass = h_k[:, :, self.rotary_dim:]            

        seq_len = h_k.shape[-2]
        sincos = self.fixed_pos_embedding(k_rot, seq_len=seq_len, dtype=h_k.dtype)
        k_rot = self.apply_rotary_pos_emb(k_rot, sincos, offset=0)
        q_rot = self.apply_rotary_pos_emb(q_rot, sincos, offset=0)

        h_q = torch.cat([q_rot, q_pass], dim=-1)
        h_k = torch.cat([k_rot, k_pass], dim=-1)
        return h_q, h_k


class SegmentPositionEmbedding(bmt.DistributedModule):

    def __init__(self, num_heads,
                       num_segments = 1,
                       num_buckets = 32, 
                       max_distance = 128, 
                       bidirectional = False, 
                       dtype = torch.half,
                       absolute_inner_segment = True,
                       init_mean : float = 0.0,
                       init_std : float = 1
                    ):

        super().__init__()
        
        self.num_heads = num_heads
        self.num_buckets = num_buckets
        self.max_distance = max_distance
        self.bidirectional = bidirectional
        self.num_segments = num_segments
        self.absolute_inner_segment = absolute_inner_segment

        self.relative_attention_bias = bmt.DistributedParameter(
            torch.empty(num_segments * num_segments + num_buckets, num_heads, dtype = dtype), 
            init_method = bmt.ParameterInitializer(nn.init.normal_, mean = init_mean, std = init_std)
        )

    def forward(self, key_pos = None, query_pos = None, key_segment = None, query_segment = None):
        """
        Args:
            key_len: int
            query_len : int
        Returns:
            out : (batch_size, num_heads, query_len, key_len)   fp16
        """
        with torch.no_grad():

            assert key_pos is not None or key_segment is not None
            assert query_pos is not None or query_segment is not None

            if isinstance(key_pos, int):
                key_pos = torch.arange(key_pos, dtype=torch.long, device="cuda")[None, :]
            if isinstance(query_pos, int):
                query_pos = torch.arange(query_pos, dtype=torch.long, device="cuda")[None, :]

            if key_pos is not None:
                batch = key_pos.size(0)
                keylen = key_pos.size(1)
                querylen = query_pos.size(1)
            else:
                batch = key_segment.size(0)
                keylen = key_segment.size(1)
                querylen = query_segment.size(1)

            if key_segment is None:
                key_segment = torch.zeros(keylen, dtype=torch.long, device="cuda")[None, :]
            if query_segment is None:
                query_segment = torch.zeros(querylen, dtype=torch.long, device="cuda")[None, :]
            if key_pos is None:
                key_pos = torch.arange(keylen, dtype=torch.long, device="cuda")[None, :]
            if query_pos is None:
                query_pos = torch.arange(querylen, dtype=torch.long, device="cuda")[None, :]

            assert key_pos.size(0) == query_pos.size(0)
            assert keylen == key_segment.size(1) and querylen == query_segment.size(1)

            key_pos = key_pos.view(batch, -1, keylen)
            query_pos = query_pos.view(batch, querylen, -1)
            key_segment = key_segment.view(batch, -1, keylen)
            query_segment = query_segment.view(batch, querylen, -1)

            relative_position_bucket = self._segment_relative_position_bucket(query_segment, key_segment)
            relative_position_bucket = relative_position_bucket + self.num_buckets  # 与相对位置编码区间不重叠
            # b*q*k
            if self.absolute_inner_segment:
                absolute_position_bucket = self._absolute_position_bucket(
                    key_pos - query_pos,
                    bidirectional=self.bidirectional,
                    num_buckets=self.num_buckets,
                    max_distance = self.max_distance
                )
                relative_position_bucket = relative_position_bucket.to(torch.int32)
                relative_position_bucket = torch.where((key_segment == query_segment), absolute_position_bucket, relative_position_bucket)
            # (batch, len_q, len_k)

        # (batch, len_q, len_k, num_heads)
        embeds = F.embedding(relative_position_bucket, self.relative_attention_bias)
        
        # (batch, num_heads, len_q, len_k)
        embeds = embeds.permute(0, 3, 1, 2).contiguous()
        return embeds

    def _segment_relative_position_bucket(self, query_segment, key_segment):
        return query_segment * self.num_segments + key_segment

    def _absolute_position_bucket(self, relative_position, bidirectional=True, num_buckets=32, max_distance=128):
        relative_buckets = 0
        if bidirectional:
            num_buckets //= 2
            relative_buckets = (relative_position > 0).to(torch.int32) * num_buckets
            relative_position = torch.abs(relative_position)
        else:
            relative_position = -torch.min(relative_position, torch.zeros_like(relative_position))
        max_exact = num_buckets // 2
        is_small = relative_position < max_exact
        relative_postion_if_large = max_exact + (torch.log(relative_position.float() / max_exact) / math.log(max_distance / max_exact) * (num_buckets - max_exact)).to(torch.int32)
        relative_postion_if_large = torch.min(relative_postion_if_large, torch.full_like(relative_postion_if_large, num_buckets - 1))
        relative_buckets += torch.where(is_small, relative_position.to(torch.int32), relative_postion_if_large)
        return relative_buckets



# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
from .config import Config

class CPM2Config(Config):
    """
    This is a configuration class that stores the configuration of the CPM-2 model, which inherits from the Config class.
    It is used to instantiate the Bert model according to the specified parameters and define the model architecture.
    You can set specific parameters to control the output of the model.

    For example:
    [`dim_model`] is used to determine the Dimension of the encoder layers.
    You can choose to use the default value of 768 or customize their dimensions.  
    
    """
    def __init__(self, vocab_size=26240,
                       dim_model=768,
                       num_heads=12,
                       dim_head=64,
                       dim_ff=1920,
                       num_encoder_layers=12,
                       num_decoder_layers=12,
                       dropout_p=0.,
                       emb_init_mean = 0.0,
                       emb_init_std = 0.02,
                       pos_bias_type="relative",
                       position_bias_num_buckets=32, 
                       position_bias_max_distance=128, 
                       pos_init_mean = 0.0,
                       pos_init_std = 1,
                       norm_init_var = 1.0,
                       norm_bias = False,
                       norm_eps = 1e-6,
                       att_init_mean = 0.0, 
                       att_init_std = 0.02,
                       att_bias = False,
                       att_mask_value = float("-inf"),
                       ffn_init_mean = 0.0, 
                       ffn_init_std = 0.02,
                       ffn_bias = False,
                       ffn_activate_fn = "gated_gelu",
                       proj_init_mean = 0.0,
                       proj_init_std = 0.02,
                       proj_bias = False,
                       length_scale = False,
                       attn_scale = False,
                       half = True, 
                       int8 = False,
                       tied = False,
                       cls_head = None,
                       post_layer_norm = False,
        ):

        super().__init__()

        self.vocab_size = vocab_size
        self.dim_model = dim_model
        self.num_heads = num_heads
        self.dim_head = dim_head
        self.dim_ff = dim_ff
        self.num_encoder_layers = num_encoder_layers
        self.num_decoder_layers = num_decoder_layers
        self.dropout_p = dropout_p
        self.emb_init_mean = emb_init_mean
        self.emb_init_std = emb_init_std
        self.pos_bias_type = pos_bias_type
        self.position_bias_num_buckets = position_bias_num_buckets
        self.position_bias_max_distance = position_bias_max_distance
        self.pos_init_mean = pos_init_mean
        self.pos_init_std = pos_init_std
        self.norm_init_var = norm_init_var
        self.norm_bias = norm_bias
        self.norm_eps = norm_eps
        self.att_init_mean = att_init_mean
        self.att_init_std = att_init_std
        self.att_bias = att_bias
        self.att_mask_value = att_mask_value
        self.ffn_init_mean = ffn_init_mean
        self.ffn_init_std = ffn_init_std
        self.ffn_bias = ffn_bias
        self.ffn_activate_fn = ffn_activate_fn
        self.proj_init_mean = proj_init_mean
        self.proj_init_std = proj_init_std
        self.proj_bias = proj_bias
        self.length_scale = length_scale
        self.attn_scale = attn_scale
        self.int8 = int8
        self.tied = tied
        if half: 
            self.dtype = torch.half
        else:
            self.dtype = torch.float
        self.cls_head = cls_head
        self.post_layer_norm = post_layer_norm


"""
from https://github.com/openai/gpt-2/, changed for chinese
"""
import json
import os
import sentencepiece as spm

"""
SentencePiece is an unsupervised text tokenizer and detokenizer mainly for Neural Network-based text generation 
systems where the vocabulary size is predetermined prior to the neural model training. SentencePiece implements 
subword units (e.g., byte-pair-encoding (BPE) [Sennrich et al.]) and unigram language model [Kudo.]) with the 
extension of direct training from raw sentences. SentencePiece allows us to make a purely end-to-end 
system that does not depend on language-specific pre/postprocessing.
https://github.com/google/sentencepiece
pip install sentencepiece
or  git clone https://github.com/google/sentencepiece.git
python setup.py install
"""

def get_pairs(word):
    pairs = set()
    prev_char = word[0]
    for char in word[1:]:
        pairs.add((prev_char, char))
        prev_char = char
    return pairs


class Encoder:
    def __init__(self, encoder, bpe_merges):
        self.encoder = encoder
        self.decoder = {v: k for k, v in self.encoder.items()}
        self.bpe_ranks = dict(zip(bpe_merges, range(len(bpe_merges))))
        self.cache = {}
        self.max_len = 0

    def bpe(self, token):
        if token in self.cache:
            return self.cache[token]
        word = tuple(token)
        pairs = get_pairs(word)
        if not pairs:
            return token

        while True:
            bigram = min(pairs, key=lambda pair: self.bpe_ranks.get(pair, float('inf')))
            if bigram not in self.bpe_ranks:
                break
            first, second = bigram
            new_word = []
            i = 0
            while i < len(word):
                try:
                    j = word.index(first, i)
                    new_word.extend(word[i:j])
                    i = j
                except:
                    new_word.extend(word[i:])
                    break

                if word[i] == first and i < len(word) - 1 and word[i + 1] == second:
                    new_word.append(first + second)
                    i += 2
                else:
                    new_word.append(word[i])
                    i += 1
            new_word = tuple(new_word)
            word = new_word
            if len(word) == 1:
                break
            else:
                pairs = get_pairs(word)
        word = ' '.join(word)
        self.cache[token] = word
        return word

    def encode(self, text):
        return [self.encoder.get(token, 1) for token in self.tokenize(text)]

    def decode(self, tokens):
        text = ''.join([self.decoder[token] for token in tokens])
        return text

    def tokenize(self, text):
        bpe_tokens = []
        bpe_tokens.extend(bpe_token for bpe_token in self.bpe(text).split(' '))
        return bpe_tokens

    def convert_tokens_to_ids(self, tokens):
        return [self.encoder.get(token, 1) for token in tokens]


class Encoder_SP:
    def __init__(self, model_path):
        self.sp = spm.SentencePieceProcessor()
        self.sp.Load(model_path)

    def encode(self, text):
        """
        text="...."
        """
        return self.sp.EncodeAsIds(text)

    def decode(self, tokens):
        """
        tokens=[x1,x2,...]
        """
        ret = []
        text = []
        for token in tokens:
            token = int(token)
            if token in [50002, 50006, 50007, 50008]:
                if text!=[]:
                    ret.append(self.sp.DecodeIds(text))
                    text = []
                if token == 50002: ret.append('[CLS]')
                if token == 50008: ret.append('[MASK]')
                if token == 50006: ret.append('[S]')
                if token == 50007: ret.append('[E]')
            else:
                text.append(token)
        if text!=[]:
            ret.append(self.sp.DecodeIds(text))

        return ''.join(ret)

    def tokenize(self, text):
        return self.sp.EncodeAsPieces(text)

    def convert_tokens_to_ids(self, tokens):
        return [self.sp.PieceToId(token) for token in tokens]

    def convert_token_to_id(self, token):
        return self.sp.PieceToId(token)

    def convert_id_to_token(self, idx):
        return self.sp.IdToPiece(idx)


def get_encoder(encoder_file, bpe_file):
    # 以下是为了同一个函数入兼容sentencepiece
    filepath, filename = os.path.split(encoder_file)
    shotname, extension = os.path.splitext(filename)

    if (".model" == extension) and (bpe_file == ""):
        return Encoder_SP(encoder_file)
    else:
        with open(encoder_file, 'r', encoding="utf-8") as f:
            encoder = json.load(f)
        with open(bpe_file, 'r', encoding="utf-8") as f:
            bpe_data = f.read()
        bpe_merges = [tuple(merge_str.split()) for merge_str in bpe_data.split('\n')[1:-1]]
        return Encoder(
            encoder=encoder,
            bpe_merges=bpe_merges,
        )

from typing import Union
from model_center.utils import check_web_and_convert_path

class GLMTokenizer:

    @classmethod
    def from_pretrained(cls, pretrained_model_name_or_path: Union[str, os.PathLike]):
        path = check_web_and_convert_path(pretrained_model_name_or_path, 'tokenizer')
        return get_encoder(os.path.join(path, 'vocab.model'), "")


import io
import os
import pickle
from typing import List
import torch
import bisect
import bmtrain as bmt

import random
import string

def _random_string():
    return ''.join(random.choices(string.ascii_uppercase + string.digits, k=8))

class FileInfo:
    def __init__(
            self,
            file_name : str,
            block_begin : int,
            block_end : int,
            nbytes : int,
            nlines : int,
            mask : bool = False
        ) -> None:
        self.file_name = file_name
        self.block_begin = block_begin
        self.block_end = block_end
        self.nbytes = nbytes
        self.nlines = nlines
        self.mask = mask
    
    @classmethod
    def _load_from_state(cls, version, data):
        if version == 1:
            file_name, block_begin, block_end, nbytes, nlines, mask = data
            return cls(file_name, block_begin, block_end, nbytes, nlines, mask)
        else:
            raise RuntimeError("Unsupported version %d" % version)

    def __reduce__(self):
        return (FileInfo._load_from_state, (1, (self.file_name, self.block_begin, self.block_end, self.nbytes, self.nlines, self.mask)))


_MASK_VALUE = 0x7fffffff
_DEFAULT_BLOCK_SIZE = 16 << 20

class DistributedDataset:
    """ Open dataset in readonly mode.
    
    `DistributeDataset` is used to read datasets in a distributed manner.
    Data in this dataset will be distributed evenly in blocks to each worker in the `distributed communicator`.
    
    **Note** When all data has been read, reading dataset again will revert back to the first data.

    Args:
        path (str): Path to dataset.
        rank (int): Rank in distributed communicator. See: bmtrain.rank()
        world_size (int): Total workers in distributed communicator. See: bmtrain.world_size()
        block_size (int): Size of each block in bytes. All files in the same dataset should have the same block size. Default: 16MB
    
    Example:
        >>> dataset = DistributedDataset("/path/to/dataset")
        >>> for i in range(10):
        >>>     dataset.read()
    """
    def __init__(self, path : str, rank : int = 0, world_size : int = 1, block_size = _DEFAULT_BLOCK_SIZE) -> None:
        # config
        self._path = path
        self._rank = rank
        self._world_size = world_size
        self._block_size = block_size

        # dataset meta
        self._block_states = torch.tensor([], dtype=torch.int)
        self._file_info : List[FileInfo] = []
        self._file_ends : List[int] = []
        self._total_blocks = 0
        self._nbytes = 0
        self._nlines = 0

        # states
        self._curr_block = None
        self._fp = None

        # cache
        self._last_mod_time = 0
        self._curr_fname = None

        self._update_states()

    def _update_states(self, fast_skip : bool = True):
        meta_path = os.path.join(self._path, "meta.bin")

        mod_time = os.stat(meta_path).st_mtime
        if self._last_mod_time < mod_time:
            # file changed
            pass
        else:
            if fast_skip:
                return

        info : List[FileInfo] = []
        if os.path.exists(meta_path):
            with open(meta_path, "rb") as f:
                info = pickle.load(f)
        
        old_len = len(self._file_info)
        if old_len > len(info):
            raise RuntimeError("Dataset meta file: changed unexpectly")
        for i in range(old_len):
            if self._file_info[i].file_name != info[i].file_name:
                raise RuntimeError("Dataset meta file: changed unexpectly")
            if self._file_info[i].block_begin != info[i].block_begin:
                raise RuntimeError("Dataset meta file: changed unexpectly")
            if self._file_info[i].block_end != info[i].block_end:
                raise RuntimeError("Dataset meta file: changed unexpectly")

        if info[0].block_begin != 0:
            raise RuntimeError("Dataset meta file: block error (0)")
        for i in range(len(info) - 1):
            if info[i].block_end != info[i + 1].block_begin:
                raise RuntimeError("Dataset meta file: block error (%d)" % (i + 1))

        if old_len == len(info) and fast_skip:
            # fast skip
            return

        if len(info) > 0:
            total_blocks = info[-1].block_end
            self._nbytes = 0
            self._nlines = 0
            for v in info:
                self._nbytes += v.nbytes
                self._nlines += v.nlines
        else:
            total_blocks = 0
            self._nbytes = 0
            self._nlines = 0
        
        if total_blocks > 0:
            masks = torch.full((total_blocks,), _MASK_VALUE, dtype=torch.int, device="cpu", requires_grad=False)
            masks[self._rank::self._world_size] = 0
            for v in info:
                if v.mask or (not os.path.exists(self._get_file_path(v.file_name))):
                    masks[v.block_begin:v.block_end] = _MASK_VALUE
            new_block_states = torch.zeros(total_blocks, dtype=torch.int, device="cpu", requires_grad=False)
            new_block_states[:self._block_states.size(0)] = self._block_states
            new_block_states = torch.maximum(new_block_states, masks)

            self._block_states = new_block_states
            
            self._file_ends = []
            for v in info:
                self._file_ends.append(v.block_end)
        else:
            self._block_states = torch.tensor([], dtype=torch.int, device="cpu", requires_grad=False)
            self._file_ends = []
        self._total_blocks = total_blocks
        self._file_info = info

        assert len(self._file_ends) == len(self._file_info)

    def _mask_file(self, f : FileInfo):
        masks = torch.full((self._total_blocks,), 0, dtype=torch.int, device="cpu", requires_grad=False)
        masks[f.block_begin:f.block_end] = _MASK_VALUE
        self._block_states = torch.maximum(self._block_states, masks)

    def _get_block_file(self, block_id : int):
        # find block in which file
        file_idx = bisect.bisect_right(self._file_ends, block_id)
        return self._file_info[file_idx]

    def _get_next_block(self):
        self._update_states()
        if self._block_states.size(0) == 0:
            raise RuntimeError("Empty dataset")
        mn_block = self._block_states.argmin().item()
        if self._block_states[mn_block].item() == _MASK_VALUE:
            raise RuntimeError("Empty dataset")
        self._block_states[mn_block] += 1
        return mn_block
    
    def state_dict(self):
        """ Returns a state dict representing the read states of the dataset.

        Example:
            >>> state = dataset.state_dict()
            >>> dataset.load_state_dict(state)
        """
        self._update_states()
        states = torch.where(
            self._block_states == _MASK_VALUE,
            torch.zeros(self._total_blocks, dtype=torch.int, device="cpu", requires_grad=False),
            self._block_states
        )

        if self._curr_block is not None:
            curr_block = self._curr_block
            curr_f = self._get_block_file(curr_block)
            inblock_offset = self._fp.tell() - (curr_block - curr_f.block_begin) * self._block_size
        else:
            curr_block = -1
            inblock_offset = 0

        with torch.no_grad():
            if self._world_size > 1:
                gpu_states = states.cuda()
                gpu_block = torch.tensor([ curr_block, inblock_offset ], dtype=torch.long).cuda()
                global_states = bmt.distributed.all_reduce(gpu_states, op="sum").cpu()
                global_block = bmt.distributed.all_gather(gpu_block).cpu()
                return {
                    "states": global_states,
                    "block": global_block
                }
            else:
                return {
                    "states": states,
                    "block": torch.tensor([[curr_block, inblock_offset]], dtype=torch.long, device="cpu")
                }
    
    def load_state_dict(self, state, strict : bool = True):
        """ Load dataset state.

        Args:
            state (dict): dataset state dict.
            strict (bool): If `strict` is True, world size needs to be the same as when exported.

        Example:
            >>> state = dataset.state_dict()
            >>> 
        """

        self._block_states = state["states"]
        self._update_states(False)

        if state["block"].size(0) != self._world_size:
            if strict:
                raise ValueError("world_size changed (%d -> %d)" % (state["block"].size(0), self._world_size))
            else:
                self._curr_block = None
                self._fp = None
                self._curr_fname = None
        else:
            curr_block = state["block"][self._rank][0].item()
            inblock_offset = state["block"][self._rank][1].item()

            if curr_block == -1:
                self._curr_block = None
            else:
                self._curr_block = curr_block
                f_info = self._get_block_file(self._curr_block)
                self._open_file(f_info.file_name, (self._curr_block - f_info.block_begin) * self._block_size + inblock_offset)
        # end

    def _get_file_path(self, fname):
        return os.path.join(self._path, fname)

    def _open_file(self, fname, offset):
        if self._curr_fname != fname:
            if self._fp is not None:
                self._fp.close()
                self._curr_fname = None
            self._fp = open(self._get_file_path(fname), "rb")
            self._curr_fname = fname
        self._fp.seek(offset, io.SEEK_SET)    # move to block

    def read(self):
        """ Read a piece of data from dataset.

        Workers in different ranks will read different data.
        """
        if self._curr_block is None:
            next_block_id = self._get_next_block()
            f_info = self._get_block_file(next_block_id)
            try:
                self._open_file(f_info.file_name, (next_block_id - f_info.block_begin) * self._block_size)
                self._curr_block = next_block_id
            except FileNotFoundError:
                self._mask_file(f_info)
                return self.read()  # read again

        MAGIC = self._fp.read(1)
        if MAGIC == b"\x1F":
            # correct
            return pickle.load(self._fp)
        elif MAGIC == b"\x00":
            # end of block
            self._curr_block = None
            return self.read()  # read next block
        else:
            raise ValueError("Invalid magic header")
    
    @property
    def nbytes(self):
        return self._nbytes


class SimpleDataset(DistributedDataset):
    def __init__(self, path: str, block_size=_DEFAULT_BLOCK_SIZE) -> None:
        super().__init__(path, 0, 1, block_size)
    
    def _get_next_block(self):
        self._update_states()
        if self._block_states.size(0) == 0:
            raise RuntimeError("Empty dataset")
        mn_block = self._block_states.argmin().item()
        if self._block_states[mn_block].item() >= 1:
            raise EOFError("no more data")
        self._block_states[mn_block] += 1
        return mn_block
    
    def __iter__(self):
        while True:
            try:
                data = self.read()
            except EOFError:
                break
            yield data
    
    def __len__(self):
        return self._nlines

class DatasetWriter:
    def __init__(self, fname, block_size):
        self._fname = fname
        self._block_size = block_size
        self._fp = open(self._fname, "wb")
        self._inblock_offset = 0

        self._nbytes = 0
        self._nlines = 0
        self._nblocks = 1
    
    def write(self, data):
        """ Write a piece of data into dataset.

        Args:
            data (Any): Serialization will be done using pickle.

        Example:
            >>> writer.write( "anything you want" )

        """
        byte_data = pickle.dumps(data)
        if self._inblock_offset + 2 + len(byte_data) > self._block_size:
            self._fp.write(b'\x00' * (self._block_size - self._inblock_offset)) # fill the remaining space with 0
            self._inblock_offset = 0
            self._nblocks += 1
            # we go to the next block
        if self._inblock_offset + 2 + len(byte_data) > self._block_size:
            raise ValueError("data is larger than block size")
        
        self._nbytes += len(byte_data)
        self._nlines += 1

        self._inblock_offset += 1 + len(byte_data)
        self._fp.write(b"\x1F")
        self._fp.write(byte_data)

    @property
    def nbytes(self):
        return self._nbytes
    
    @property
    def nblocks(self):
        return self._nblocks
    
    @property
    def nlines(self):
        return self._nlines

    def close(self):
        if not self._fp.closed:
            self._fp.write(b"\x00" * (self._block_size - self._inblock_offset)) 
            self._fp.close()


class DatasetBuilder:
    def __init__(self, path : str, dbname : str, block_size = _DEFAULT_BLOCK_SIZE) -> None:
        self._block_size = block_size
        self._path = path
        self._dbname = dbname

        if not os.path.exists(self._path):
            os.makedirs(self._path)

        meta_path = os.path.join(self._path, "meta.bin")

        info : List[FileInfo] = []
        if os.path.exists(meta_path):
            with open(meta_path, "rb") as f:
                info = pickle.load(f)
        
        for v in info:
            if v.file_name == dbname:
                raise ValueError("Dataset name exists")
            
        self._db_path = os.path.join(self._path, self._dbname)
        if os.path.exists(self._db_path):
            raise ValueError("File exists `%s`" % self._db_path)
    
    def __enter__(self):
        self._writer = DatasetWriter(self._db_path, self._block_size)
        return self._writer
    
    def __exit__(self, exc_type, exc_value, exc_traceback):
        self._writer.close()
        if exc_type is not None:
            print("Error while writing file")
            if os.path.exists(self._db_path):
                os.unlink(self._db_path)
        else:
            meta_path = os.path.join(self._path, "meta.bin")
            info : List[FileInfo] = []
            if os.path.exists(meta_path):
                with open(meta_path, "rb") as f:
                    info = pickle.load(f)
            last_block = 0
            if len(info) > 0:
                last_block = info[-1].block_end
            info.append(FileInfo(
                self._dbname,
                last_block,
                last_block + self._writer.nblocks,
                self._writer.nbytes,
                self._writer.nlines,
                False
            ))

            # atomic write to meta file
            random_fname = os.path.join(self._path, ".meta.bin.%s" % _random_string())
            with open(random_fname, "wb") as f:
                pickle.dump(info, f)
            os.rename(random_fname, meta_path)
        
        self._writer = None

def build_dataset(path : str, dbname : str, block_size : int = _DEFAULT_BLOCK_SIZE):
    """ Open the dataset in write mode and returns a writer.

    Args:
        path (str): Path to dataset.
        dbname (str): The name of the file to which the data will be written. The `dbname` needs to be unique in this `dataset`.
        block_size (int): Size of each block in bytes. All files in the same dataset should have the same block size. Default: 16MB
    
    Example:
        >>> with build_dataset("/path/to/dataset", "data_part_1") as writer:
        >>>     for i in range(10):
        >>>         writer.write( { "anything you want" } )
    """
    return DatasetBuilder(path, dbname, block_size)



# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
from .config import Config

class VitConfig(Config):
    """
    This is a configuration class that stores the configuration of the Vit model, which inherits from the Config class.
    It is used to instantiate the vit model according to the specified parameters and define the model architecture.
    You can set specific parameters to control the output of the model.

    For example:
    [`hidden_size`] is used to determine the Dimension of the encoder layers.
    You can choose to use the default value of 768 or customize their dimensions.  
    
    """
    def __init__(self, img_size=224,
                       patch_size=16,
                       channels_in=3,
                       num_classes=1000,
                       hidden_size=768,
                       num_layers=12,
                       num_heads=12,
                       mlp_size=3072,
                       attn_bias=True,
                       attn_scale=None,
                       norm_bias=True,
                       ffn_bias=True,
                       representation_size=None,
                       drop=0.,
                       half=True,
                       dtype=torch.float):

        super().__init__()

        self.img_size = img_size
        self.patch_size = patch_size
        self.channels_in = channels_in
        self.num_classes = num_classes
        self.hidden_size = hidden_size
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.mlp_size = mlp_size
        self.attn_bias = attn_bias
        self.attn_scale = attn_scale
        self.norm_bias = norm_bias
        self.ffn_bias = ffn_bias
        self.representation_size = representation_size
        self.drop = drop
        if half:
            self.dtype = torch.half
        else:
            self.dtype = torch.float


# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch

from .attention import Attention, SparseSelfAttention
from .layernorm import LayerNorm
from .feedforward import FeedForward
import bmtrain as bmt
from typing import *


class SelfAttentionBlock(torch.nn.Module):
    """  The whole cross-attention block. A sequence of operation. Consists of layernorm, self-attention and residual connection.

    Args:
        dim_model (int): main dimension of modules in transformer blocks.
        num_heads (int): num_heads used in :py:class:`model_center.layer.Attention`.
        dim_head (int): dim_head used in :py:class:`model_center.layer.Attention`.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        att_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.Attention`. Defaults to 0.0.
        att_init_std (float, optional): init_std used in :py:class:`model_center.layer.Attention`. Defaults to 0.02.
        att_bias (bool, optional): bias used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        att_mask_value (float, optional): mask_value used in in :py:class:`model_center.layer.Attention`. Defaults to float("-inf").
        pos_bias_type (str, optional): pos_bias_type used in :py:class:`model_center.layer.Attention`. Defaults to "none".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        attn_scale (bool, optional): attn_scale used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        dropout_p (float, optional): Defaults to 0.
    """

    def __init__(self, 
                 dim_model : int, 
                 num_heads : int, 
                 dim_head : int, 
                 dtype = torch.half,
                 int8 = False, 
                 norm_init_var : float = 1.0,
                 norm_bias : bool = False,
                 norm_eps : float = 1e-5, 
                 att_init_mean : float = 0.0, 
                 att_init_std : float = 0.02,
                 att_bias : bool = False,
                 att_mask_value : float = float("-inf"),
                 pos_bias_type : str = "none",
                 post_layer_norm : bool = False,
                 length_scale : bool = False,
                 attn_scale : bool = False,
                 dropout_p : float = 0,
                 sparse_attention : bool = False,
                 attention_window : int = 512,
        ):

        super().__init__()

        self.layernorm_before_attention = LayerNorm(
            dim_norm = dim_model, 
            bias = norm_bias, 
            dtype = dtype,
            eps = norm_eps, 
            init_var = norm_init_var,
        )
        self.sparse_attention = sparse_attention
        if not sparse_attention:
            self.self_attention = Attention(
                dim_in = dim_model, 
                num_heads = num_heads, 
                dim_head = dim_head,
                dim_out = dim_model, 
                dtype = dtype,
                int8 = int8, 
                init_mean = att_init_mean,
                init_std = att_init_std,
                bias = att_bias,
                mask_value = att_mask_value,
                pos_bias_type = pos_bias_type,
                length_scale = length_scale,
                attn_scale = attn_scale,
                dropout_p = dropout_p,
            )
        else:
            self.self_attention = SparseSelfAttention(
                dim_in = dim_model, 
                num_heads = num_heads, 
                dim_head = dim_head,
                dim_out = dim_model, 
                dtype = dtype,
                int8 = int8, 
                init_mean = att_init_mean,
                init_std = att_init_std,
                bias = att_bias,
                mask_value = att_mask_value,
                pos_bias_type = pos_bias_type,
                length_scale = length_scale,
                attn_scale = attn_scale,
                dropout_p = dropout_p,
                attention_window = attention_window,
            )
        if dropout_p:
            self.dropout = torch.nn.Dropout(dropout_p)
        else:
            self.dropout = None

        self.post_layer_norm = post_layer_norm

    def forward(self,
                hidden_states : torch.Tensor,
                attention_mask : torch.Tensor,
                position_bias : Optional[torch.Tensor] = None,
                use_cache : bool = False,
                past_key_value = None,
            ):
        """
        Args:
            hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``): Input of self-attention block. It can be the embedding of a batch of sequences.
            attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_self, seq_self)``): Avoid invalid areas to participate in the calculation.  
            position_bias (:obj:`torch.Tensor` of shape ``(num_heads, seq_self, seq_self)``): Provide positional information to self-attention block.

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``: The output of attention block.

        """    
        x = self.layernorm_before_attention(hidden_states)
        if self.post_layer_norm:
            hidden_states = x
        if not self.sparse_attention:
            x = self.self_attention(x, x, attention_mask, position_bias, use_cache, past_key_value)
        else:
            #no position bias for sparse attention
            x = self.self_attention(x, attention_mask, position_bias)

        if use_cache:
            x, current_key_value = x
        else:
            current_key_value = None

        if self.dropout is not None:
            x = self.dropout(x)
        hidden_states = hidden_states + x

        if use_cache:
            return hidden_states, current_key_value
        else:
            return hidden_states


class CrossAttentionBlock(torch.nn.Module):
    """  The whole cross-attention block. A sequence of operation. Consists of layernorm, cross-attention and residual connection.

    Args:
        dim_model (int): main dimension of modules in transformer blocks.
        num_heads (int): num_heads used in :py:class:`model_center.layer.Attention`.
        dim_head (int): dim_head used in :py:class:`model_center.layer.Attention`.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        att_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.Attention`. Defaults to 0.0.
        att_init_std (float, optional): init_std used in :py:class:`model_center.layer.Attention`. Defaults to 0.02.
        att_bias (bool, optional): bias used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        att_mask_value (float, optional): mask_value used in in :py:class:`model_center.layer.Attention`. Defaults to float("-inf").
        pos_bias_type (str, optional): pos_bias_type used in :py:class:`model_center.layer.Attention`. Defaults to "none".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        attn_scale (bool, optional): attn_scale used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        dropout_p (float, optional): Defaults to 0.
    """

    def __init__(self, 
                 dim_model : int, 
                 num_heads : int, 
                 dim_head : int, 
                 dtype = torch.half,
                 int8 = False, 
                 norm_init_var : float = 1.0,
                 norm_bias : bool = False,
                 norm_eps : float = 1e-5, 
                 att_init_mean : float = 0.0, 
                 att_init_std : float = 0.02,
                 att_bias : bool = False,
                 att_mask_value : float = float("-inf"),
                 pos_bias_type : str = "none",
                 post_layer_norm : bool = False,
                 length_scale : bool = False,
                 attn_scale : bool = False,
                 dropout_p : float = 0,
                 sparse_attention : bool = False,
                 attention_window : int = 512,
        ):

        super().__init__()

        self.layernorm_before_attention = LayerNorm(
            dim_norm = dim_model, 
            bias = norm_bias, 
            dtype = dtype,
            eps = norm_eps, 
            init_var = norm_init_var,
        )

        self.sparse_attention = sparse_attention
        if not sparse_attention:
            self.self_attention = Attention(
                dim_in = dim_model, 
                num_heads = num_heads, 
                dim_head = dim_head,
                dim_out = dim_model, 
                dtype = dtype,
                int8 = int8, 
                init_mean = att_init_mean,
                init_std = att_init_std,
                bias = att_bias,
                mask_value = att_mask_value,
                pos_bias_type = pos_bias_type,
                length_scale = length_scale,
                attn_scale = attn_scale,
                dropout_p = dropout_p,
            )
        else:
            self.self_attention = SparseSelfAttention(
                dim_in = dim_model, 
                num_heads = num_heads, 
                dim_head = dim_head,
                dim_out = dim_model, 
                dtype = dtype,
                int8 = int8, 
                init_mean = att_init_mean,
                init_std = att_init_std,
                bias = att_bias,
                mask_value = att_mask_value,
                pos_bias_type = pos_bias_type,
                length_scale = length_scale,
                attn_scale = attn_scale,
                dropout_p = dropout_p,
                attention_window = attention_window,
            )

        if dropout_p:
            self.dropout = torch.nn.Dropout(dropout_p)
        else:
            self.dropout = None

        self.post_layer_norm = post_layer_norm

    def forward(self,
                hidden_states : torch.Tensor,
                key_value_states: torch.Tensor,
                attention_mask : torch.Tensor,
                position_bias : Optional[torch.Tensor] = None,
                use_cache : bool = False,
                past_key_value = None,
            ):
        """
        Args:
            hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``): Input of cross-attention block. It can be seen as query in the coming self-attention operation.
            key_value_states(:obj:`torch.Tensor` of shape ``(batch, seq_cross, dim_model)``): Used as key_value in coming self_attention operation. 
            attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_self, seq_cross)``): Avoid invalid areas to participate in the calculation.  
            position_bias (:obj:`torch.Tensor` of shape ``(num_heads, seq_self, seq_cross)``): Provide positional information to self-attention block.

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``: The output of cross-attention block.

        """ 
        x = self.layernorm_before_attention(hidden_states)
        if self.post_layer_norm:
            hidden_states = x

        if not self.sparse_attention:
            x = self.self_attention(x, key_value_states, attention_mask, position_bias, use_cache, past_key_value)
        else:
            #no position bias for sparse attention
            #to do
            x = self.self_attention(x, attention_mask, position_bias)

        if use_cache:
            x, current_key_value = x
        else:
            current_key_value = None

        if self.dropout is not None:
            x = self.dropout(x)
        hidden_states = hidden_states + x

        if use_cache:
            return hidden_states, current_key_value
        else:
            return hidden_states


class FFNBlock(torch.nn.Module):
    """ The whole feed-forward block. A sequence of operation. Consists of layernorm, feed-forward and residual connection.

    Args:
        dim_model (int): main dimension of modules in transformer blocks.
        dim_ff (int): dim_ff used in :py:class:`model_center.layer.FeedForward`.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        ffn_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.0.
        ffn_init_std (float, optional): init_std used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.02.
        ffn_bias (bool, optional): bias used in :py:class:`model_center.layer.FeedForward`. Defaults to False.
        ffn_activate_fn (str, optional): activate_fn used in :py:class:`model_center.layer.FeedForward`. Defaults to "gated_gelu".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        dropout_p (float, optional): Defaults to 0.
    """

    def __init__(self, 
                 dim_model : int, 
                 dim_ff : int,
                 dtype = torch.half, 
                 int8 = False,
                 norm_init_var : float = 1.0,
                 norm_bias : bool = False,
                 norm_eps : float = 1e-5, 
                 ffn_init_mean : float = 0.0, 
                 ffn_init_std : float = 0.02,
                 ffn_bias : bool = False,
                 ffn_activate_fn : str = "gated_gelu",
                 post_layer_norm : bool = False,
                 length_scale : bool = False,
                 dropout_p : float = 0,
                ):
        super().__init__()

        self.layernorm_before_ffn = LayerNorm(
            dim_norm = dim_model, 
            bias = norm_bias, 
            dtype = dtype,
            eps = norm_eps, 
            init_var = norm_init_var,
        )

        self.ffn = FeedForward(
            dim_in = dim_model, 
            dim_ff = dim_ff, 
            dim_out = dim_model, 
            dtype = dtype, 
            int8 = int8,
            init_mean = ffn_init_mean, 
            init_std = ffn_init_std,
            bias = ffn_bias,
            activate_fn = ffn_activate_fn,
            length_scale = length_scale,
        )

        if dropout_p:
            self.dropout = torch.nn.Dropout(dropout_p)
        else:
            self.dropout = None

        self.post_layer_norm = post_layer_norm

    def forward(self,
                hidden_states : torch.Tensor,
               ):
        """ 
        Args:
            hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``): Hidden states before feed forward layer.

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``: The output of feed-forward block

        """ 
        x = self.layernorm_before_ffn(hidden_states)
        if self.post_layer_norm:
            hidden_states = x
        x = self.ffn(x)
        if self.dropout is not None:
            x = self.dropout(x)
        hidden_states = hidden_states + x
        return hidden_states


class TransformerBlock(torch.nn.Module):
    """ The whole transformer block. A sequence of operation. Consists of self-attention block[, cross-attention block] and feed-forward block.

    Args:
        dim_model (int): main dimension of modules in transformer blocks.
        dim_ff (int): dim_ff used in :py:class:`model_center.layer.FeedForward`.
        num_heads (int): num_heads used in :py:class:`model_center.layer.Attention`.
        dim_head (int): dim_head used in :py:class:`model_center.layer.Attention`.
        is_decoder (bool, optional): whether to use cross-attention. Defaults to False.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        att_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.Attention`. Defaults to 0.0.
        att_init_std (float, optional): init_std used in :py:class:`model_center.layer.Attention`. Defaults to 0.02.
        att_bias (bool, optional): bias used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        att_mask_value (float, optional): mask_value used in in :py:class:`model_center.layer.Attention`. Defaults to float("-inf").
        ffn_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.0.
        ffn_init_std (float, optional): init_std used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.02.
        ffn_bias (bool, optional): bias used in :py:class:`model_center.layer.FeedForward`. Defaults to False.
        ffn_activate_fn (str, optional): activate_fn used in :py:class:`model_center.layer.FeedForward`. Defaults to "gated_gelu".
        pos_bias_type (str, optional): pos_bias_type used in :py:class:`model_center.layer.Attention`. Defaults to "none".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        attn_scale (bool, optional): attn_scale used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        dropout_p (float, optional): Defaults to 0.
    """

    def __init__(self, 
                 dim_model : int, 
                 dim_ff : int,
                 num_heads : int,
                 dim_head : int,
                 is_decoder : bool = False,
                 dtype = torch.half, 
                 int8 = False,
                 norm_init_var : float = 1.0,
                 norm_bias : bool = False,
                 norm_eps : float = 1e-5, 
                 att_init_mean : float = 0.0, 
                 att_init_std : float = 0.02,
                 att_bias : bool = False,
                 att_mask_value : float = float("-inf"),
                 ffn_init_mean : float = 0.0, 
                 ffn_init_std : float = 0.02,
                 ffn_bias : bool = False,
                 ffn_activate_fn : str = "gated_gelu",
                 pos_bias_type : str = "none",
                 post_layer_norm : bool = False,
                 parallel_ffn : bool = False,
                 length_scale : bool = False,
                 attn_scale : bool = False,
                 dropout_p : float = 0,
                 sparse_attention : bool = False,
                 attention_window : int = 512,
                 mask_att: bool = False,
                 mask_cross: bool = False,
                 mask_ffn: bool = False,
                ):
        super().__init__()

        self.mask_att = mask_att
        self.mask_cross = mask_cross
        self.mask_ffn = mask_ffn
        self.is_decoder = is_decoder

        if not mask_att:
            self.self_att = SelfAttentionBlock(
                dim_model = dim_model, 
                num_heads = num_heads, 
                dim_head = dim_head, 
                dtype = dtype,
                int8 = int8, 
                norm_eps = norm_eps, 
                norm_init_var = norm_init_var,
                norm_bias = norm_bias,
                att_init_mean = att_init_mean, 
                att_init_std = att_init_std,
                att_bias = att_bias,
                att_mask_value = att_mask_value,
                pos_bias_type = pos_bias_type,
                post_layer_norm = post_layer_norm,
                length_scale = length_scale,
                attn_scale = attn_scale,
                dropout_p = dropout_p,
                sparse_attention = sparse_attention,
                attention_window = attention_window,
            )

        if is_decoder and not mask_cross:
            self.cross_att = CrossAttentionBlock(
                dim_model = dim_model, 
                num_heads = num_heads, 
                dim_head = dim_head, 
                dtype = dtype,
                int8 = int8, 
                norm_eps = norm_eps, 
                norm_init_var = norm_init_var,
                norm_bias = norm_bias,
                att_init_mean = att_init_mean, 
                att_init_std = att_init_std,
                att_bias = att_bias,
                att_mask_value = att_mask_value,
                pos_bias_type = pos_bias_type,
                length_scale = length_scale,
                attn_scale = attn_scale,
                dropout_p = dropout_p,
            )
        else:
            self.cross_att = None

        if not mask_ffn:
            self.ffn = FFNBlock(
                dim_model = dim_model, 
                dim_ff = dim_ff,
                dtype = dtype, 
                int8 = int8,
                norm_eps = norm_eps, 
                norm_init_var = norm_init_var,
                norm_bias = norm_bias,
                ffn_init_mean = ffn_init_mean, 
                ffn_init_std = ffn_init_std,
                ffn_bias = ffn_bias,
                ffn_activate_fn = ffn_activate_fn,
                length_scale = length_scale,
                dropout_p = dropout_p,
                post_layer_norm = post_layer_norm,
            )

        self.parallel_ffn = parallel_ffn

    def forward(self,
                self_hidden_states : torch.Tensor,
                self_attention_mask : torch.Tensor,
                self_position_bias : Optional[torch.Tensor] = None,
                cross_hidden_states = None,
                cross_attention_mask = None,
                cross_position_bias = None,
                use_cache : bool = False,
                past_key_value = None,
            ):
        """
        Args:
            self_hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``): Input of transformer block(self-attention block). It can be the raw embedding of a batch of sequences.
            self_attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_self, seq_self)``): Avoid invalid areas to participate in the calculation of self-attention.  
            self_position_bias (:obj:`torch.Tensor` of shape ``(num_heads, seq_self, seq_self)``): Provide positional information to self-attention block.
            cross_hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_cross, dim_model)``): Input of cross-attention block. 
            cross_attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_self, seq_cross)``): Avoid invalid areas to participate in the calculation of cross-attention.  
            cross_position_bias (:obj:`torch.Tensor` of shape ``(num_heads, seq_self, seq_cross)``): Provide positional information to cross-attention block.

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``: The output of transformer block.

        """
        current_key_value = None
        if not self.mask_att:
            # (batch, dim_model, seq_self)
            # add positional bias on sparse attention in the future
            hidden_states = self.self_att(self_hidden_states,
                                        attention_mask = self_attention_mask,
                                        position_bias = self_position_bias,
                                        use_cache = use_cache,
                                        past_key_value = past_key_value)
            if use_cache:
                hidden_states, current_key_value = hidden_states
        else:
            hidden_states = self_hidden_states

        if self.is_decoder and self.cross_att is not None:
            if not self.mask_cross:
                # (batch, dim_model, seq_self)
                hidden_states = self.cross_att(hidden_states = hidden_states,
                                            key_value_states = cross_hidden_states,
                                            attention_mask = cross_attention_mask,
                                            position_bias = cross_position_bias)

        if not self.mask_ffn:
            # (batch, dim_model, seq_self)
            if self.parallel_ffn:
                hidden_states_2 = self.ffn(self_hidden_states)
                hidden_states = hidden_states - self_hidden_states + hidden_states_2
            else:
                hidden_states = self.ffn(hidden_states)

        if use_cache:
            return hidden_states, current_key_value
        else:
            return hidden_states




# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
from ..layer import Encoder, Decoder, Linear, Embedding, RelativePositionEmbedding, LayerNorm
from .basemodel import BaseModel
from .config import GLMConfig

class GLM(BaseModel):

    _CONFIG_TYPE = GLMConfig
    
    def __init__(self, config: GLMConfig):
        
        super().__init__()
        self.sop_tok_id = config.sop_tok_id
        self.mask_tok_id = config.mask_tok_id
        self.encoder = Encoder(
            num_layers = config.num_layers,
            dim_model = config.dim_model, 
            dim_ff = config.dim_ff,
            num_heads = config.num_heads,
            dim_head = config.dim_head,
            dtype = config.dtype, 
            int8 = config.int8,
            norm_eps = config.norm_eps, 
            norm_init_var = config.norm_init_var,
            norm_bias = config.norm_bias,
            att_init_mean = config.att_init_mean, 
            att_init_std = config.att_init_std,
            att_bias = config.att_bias,
            att_mask_value = float(config.att_mask_value),
            pos_bias_type = config.pos_bias_type,
            ffn_init_mean = config.ffn_init_mean, 
            ffn_init_std = config.ffn_init_std,
            ffn_bias = config.ffn_bias,
            ffn_activate_fn = config.ffn_activate_fn,
            length_scale = config.length_scale,
            attn_scale = config.attn_scale,
            dropout_p = config.dropout_p,
        )

        self.input_embedding = Embedding(
            vocab_size = config.vocab_size, 
            embedding_size = config.dim_model,
            length_scale = config.length_scale,
            dtype = config.dtype,
            int8 = config.int8,
            init_mean = config.emb_init_mean,
            init_std = config.emb_init_std,
        )

        self.position_embedding = Embedding(
            vocab_size = config.position_size, 
            embedding_size = config.dim_model,
            length_scale = config.length_scale,
            dtype = config.dtype,
            int8 = config.int8,
            init_mean = config.emb_init_mean,
            init_std = config.emb_init_std,
        )

        self.block_position_embedding = Embedding(
            vocab_size = config.position_size, 
            embedding_size = config.dim_model,
            length_scale = config.length_scale,
            dtype = config.dtype,
            int8 = config.int8,
            init_mean = config.emb_init_mean,
            init_std = config.emb_init_std,
        )

        self.embed_dropout = torch.nn.Dropout(config.dropout_p)
        
        self.tied = config.tied
        self.cls_head = config.cls_head
        if self.cls_head:
            self.output_projection = Linear(
                dim_out = self.cls_head,
                dim_in = config.dim_model,
                length_scale = config.length_scale,
                dtype = config.dtype,
                int8 = config.int8,
                init_mean = config.proj_init_mean,
                init_std = config.proj_init_std,
                bias = config.proj_bias,
            )

    def forward(self, input_ids : torch.Tensor, # (batch, seqlen)
                      position_ids : torch.Tensor = None, # (batch, seqlen)
                      block_position_ids : torch.Tensor = None, # (batch, seqlen)
                      sep : torch.Tensor = None, # (batch)
    ):

        batch = input_ids.size(0)
        seq_len = input_ids.size(1)
        device = input_ids.device
        if sep is None:
            sep = torch.empty(batch, dtype=torch.long, device=device)
            for b in range(batch):
                sep_idx = (input_ids[b]==self.sop_tok_id).nonzero()
                if min(sep_idx.shape) > 0:
                    sep[b] = sep_idx[0]
                else:
                    sep[b] = seq_len
        with torch.no_grad():
            mask_1d = torch.arange(seq_len, device=device)[None, :].repeat(batch, 1) < sep[:, None]
            directional_mask_2d = torch.arange(seq_len, device=device) <= torch.arange(seq_len, device=device).view(-1, 1)
            attention_mask = mask_1d.view(batch, 1, seq_len) | directional_mask_2d.view(1, seq_len, seq_len)

        hidden_states = self.input_embedding(input_ids)
        if position_ids is None:
            position_ids = torch.arange(seq_len, device=device)[None, :].repeat(batch, 1)
            for b in range(batch):
                head = sep[b]
                mask_idx = (input_ids[b]==self.mask_tok_id).nonzero() 
                if min((input_ids[b] == self.sop_tok_id).nonzero().shape) > 0:
                    for idx,tail in enumerate((input_ids[b] == self.sop_tok_id).nonzero()[1:]):
                        position_ids[b][head:tail] = mask_idx[idx] 
                        head = tail
                if head != seq_len and min(mask_idx.shape)>0:
                    position_ids[b][head:] = mask_idx[-1]
        if block_position_ids is None:
            block_position_ids = torch.zeros((batch, seq_len), device=device, dtype=torch.long)
            for b in range(batch):
                head = sep[b]
                if min((input_ids[b] == self.sop_tok_id).nonzero().shape) > 1:
                    for tail in (input_ids[b] == self.sop_tok_id).nonzero()[1:]:
                        block_position_ids[b][head:tail] = torch.arange(1, 1+tail-head, device=device) 
                        head = tail
                if head != seq_len:
                    block_position_ids[b][head:] = torch.arange(1, 1+seq_len-head, device=device)
        position_embedding = self.position_embedding(position_ids)
            
        position_embeds = self.position_embedding(position_ids)
        hidden_states = hidden_states + position_embeds
        
        block_position_embeds = self.block_position_embedding(block_position_ids)
        hidden_states = hidden_states + block_position_embeds

        hidden_states = self.embed_dropout(hidden_states)

        hidden_states = self.encoder(hidden_states, attention_mask)

        if self.cls_head:
            logits = self.output_projection(hidden_states)
        else:
            logits = self.input_embedding.projection(hidden_states)

        return logits


# coding=utf-8
# Copyright 2018 The Open AI Team Authors and The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This code is adapted from OpenAI Megatron for CPM2 tokenizer"""
# TODO will change to our SAM implementation in the future

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from io import open
import jieba
import collections
import six
import os
from typing import Union

from model_center.utils import check_web_and_convert_path

try:
    from functools import lru_cache
except ImportError:
    # Just a dummy decorator to get the checks to run on python2
    # because honestly I don't want to support a byte-level unicode BPE tokenizer on python 2 right now.
    def lru_cache():
        return lambda func: func

def convert_to_unicode(text):
  """Converts `text` to Unicode (if it's not already), assuming utf-8 input."""
  if six.PY3:
    if isinstance(text, str):
      return text
    elif isinstance(text, bytes):
      return text.decode("utf-8", "ignore")
    else:
      raise ValueError("Unsupported string type: %s" % (type(text)))
  elif six.PY2:
    if isinstance(text, str):
      return text.decode("utf-8", "ignore")
    elif isinstance(text, unicode):
      return text
    else:
      raise ValueError("Unsupported string type: %s" % (type(text)))
  else:
    raise ValueError("Not running on Python2 or Python 3?")

def load_vocab(vocab_file):
  """Loads a vocabulary file into a dictionary."""
  vocab = collections.OrderedDict()
  index = 0
  with open(vocab_file, "r", encoding="utf-8") as reader:
    while True:
      token = convert_to_unicode(reader.readline())
      if not token:
        break
      token = token.strip()
      vocab[token] = index
      index += 1
  return vocab

def is_contain_chinese(check_str):
    for ch in check_str:
        if u'\u4e00' <= ch <= u'\u9fff':
            return True
    return False

def is_contain_point(check_str):
    for ch in check_str:
        if u'0' <= ch <= u'9':
            return True
    return False


class WordpieceTokenizer(object):

    def __init__(self, vocab, unk_token="<unk>", max_input_chars_per_word=200):
        self.vocab = vocab
        self.unk_token = unk_token
        self.max_input_chars_per_word = max_input_chars_per_word

    def tokenize(self, token):

        token = convert_to_unicode(token)

        chars = list(token)
        if len(chars) > self.max_input_chars_per_word:
            return [self.unk_token]

        start = 0
        sub_tokens = []
        while start < len(chars):
            end = len(chars)
            cur_substr = None
            while start < end:
                substr = "".join(chars[start:end])
                if is_contain_chinese(substr) or is_contain_point(substr):
                    if substr in self.vocab:
                        cur_substr = substr
                        break
                else:
                    # if start > 0:
                    #     substr = "##" + substr
                    if substr in self.vocab:
                        cur_substr = substr
                        break
                end -= 1
            if cur_substr is None:
                sub_tokens.append(self.unk_token)
                start += 1
                continue
            sub_tokens.append(cur_substr)
            start = end

        return sub_tokens


class CPM2Tokenizer(object): # TODO

    @classmethod
    def from_pretrained(cls, pretrained_model_name_or_path: Union[str, os.PathLike]):
        path = check_web_and_convert_path(pretrained_model_name_or_path, 'tokenizer')
        return cls(os.path.join(path, 'vocab.txt'), space_token = "▂", line_token = "▃")

    def __init__(self, 
                 vocab_file, 
                 max_sentinels = 190, 
                 max_len = None, 
                 q2b = False,
                 sod_token = '<s>',
                 eod_token = '<eod>',
                 pad_token = '<pad>',
                 unk_token = '<unk>',
                 line_token = '</n>',
                 space_token = '</_>'):

        self.sod_token = sod_token
        self.eod_token = eod_token
        self.pad_token = pad_token
        self.unk_token = unk_token
        self.line_token = line_token
        self.space_token = space_token

        self.max_len = max_len if max_len is not None else int(1e12)

        self.encoder = load_vocab(vocab_file)
        self.encoder[self._space_token] = len(self.encoder)
        self.encoder[self._line_token] = len(self.encoder)
        self.decoder = {v : k for k, v in self.encoder.items()}

        self.wordpiece_tokenizer = WordpieceTokenizer(vocab = self.encoder, 
                                                      unk_token = self.unk_token)

        self.trans_common = str.maketrans(" \n", "\u2582\u2583")
        BAN1='｡､｢｣'
        QUAN1='。、「」'
        # 全角转半角
        QUAN2='\u3000＂＇＾｀，：；？！「」『』（）｛｝［］〔〕＜＞〜｜＼＿＄％＃＆＠＝＊／＋－．０１２３４５６７８９ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ'
        BAN2 =' "\'^`,:;?!“”‘’(){}[][]<>~|\_$%#&@=*/+-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
        # 暂不转换的全半角
        # FULL = " ゛´—‐‘’“”〈〉′″"
        # half='''
        #  "'--''""[]
        # '''
        self.trans_qb_1 = str.maketrans(QUAN1, BAN1)
        self.trans_qb_2 = str.maketrans(QUAN2, BAN2)
        self.q2b = q2b

        self.sentinel_list = [
            self.encoder['<s_{}>'.format(i)] 
                for i in range(max_sentinels)]

        self.en_vocab = {}
        for k, v in self.encoder.items():
            if is_contain_chinese(k):
                self.en_vocab[v] = False
            else:
                self.en_vocab[v] = True

        self._space_id = self.encoder[self._space_token]
        self.en_vocab[self._space_id] = False

        self._line_id = self.encoder[self._line_token]
        self.en_vocab[self._line_id] = False

        self.space_id = self.encoder[self.space_token]
        self.line_id = self.encoder[self.line_token]

    @property
    def vocab_size(self):
        return len(self.encoder) - 2

    @property
    def sod_id(self):
        return self.encoder[self.sod_token]

    @property
    def eod_id(self):
        return self.encoder[self.eod_token]

    @property
    def pad_id(self):
        return self.encoder[self.pad_token]

    @property
    def unk_id(self):
        return self.encoder[self.unk_token]

    @property
    def _space_token(self):
        return '\uFFF2'

    @property
    def _line_token(self):
        return '\uFFF3'

    def __len__(self):
        return len(self.encoder)

    def get_sentinel_num(self):
        return len(self.sentinel_list)

    def get_sentinel_id(self, idx):
        return self.sentinel_list[idx]

    def _translate(self, sent):
        if self.q2b:
            return sent.translate(self.trans_common) \
                       .translate(self.trans_qb_1) \
                       .translate(self.trans_qb_2)
        else:
            return sent.translate(self.trans_common)

    def tokenize(self, text):
        """ Tokenize a string. """
        output_tokens = []
        for x in jieba.cut(text, cut_all=False):
            output_tokens.extend(
                self.wordpiece_tokenizer.tokenize(
                    self._translate(x)
                )
            )
        return output_tokens

    def encode(self, text):
        """ Encode a string into ids. """
        text = text.replace("\n", self._line_token) \
                   .replace(self.line_token, self._line_token) \
                   .replace(" ", self._space_token) \
                   .replace(self.space_token, self._space_token)
        new_output_tokens = []
        for x in self.tokenize(text):
            x = self.encoder[x]
            if x == self._space_id:
                new_output_tokens.append(self.space_id)
            elif x == self._line_id:
                new_output_tokens.append(self.line_id)
            else:
                new_output_tokens.append(x)
        return new_output_tokens

    def decode(self, tokens):
        """ Decode ids into a string. """
        text = ''.join([self.decoder[x] for x in tokens])
        text = text.replace(self._space_token, ' ') \
                   .replace(self._line_token, '\n') \
                   .replace(self.space_token, ' ') \
                   .replace(self.line_token, '\n')
        return text

    def check(self, token):
        return token in self.encoder

    def convert_tokens_to_ids(self, tokens):
        return [self.encoder.get(x, self.encoder[self.unk_token]) for x in tokens]

    def convert_ids_to_tokens(self, ids):
        _ids = ids.cpu().numpy()
        return [self.decoder[x] for x in _ids]



from setuptools import setup, find_packages
import os

def main():
    setup(
        name='model-center',
        version='0.1.5',
        description="example codes for big models using bmtrain",
        author="Weilin Zhao",
        author_email="acha131441373@gmail.com",
        packages=find_packages(),
        url="https://github.com/OpenBMB/BModelCenter",
        install_requires=[
            "bmtrain",
            "transformers",
            "jieba",
        ],
        keywords="CPM, cuda, AI, model, transformer",
        license='Apache 2.0',
    )

if __name__ == '__main__':
    main()



# coding=utf-8
# Copyright 2020 The OpenBMB team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import torch.utils.data as data
import bmtrain as bmt

class DistributedDataLoader:
    def __init__(self, dataset, shuffle=False, **kwargs):
        self.sampler = data.distributed.DistributedSampler(dataset, shuffle=shuffle, rank=bmt.rank(), num_replicas=bmt.world_size())
        self.loader = data.DataLoader(dataset, shuffle=False, sampler=self.sampler, **kwargs)
        self.epoch = 0
        self.shuffle = shuffle

    def __iter__(self):
        if self.shuffle:
            self.epoch += 1
        self.sampler.set_epoch(self.epoch)
        return self.loader.__iter__()

    def __len__(self):
        return len(self.loader)
    


# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
import bmtrain as bmt
from typing import Optional, List, Tuple

from .blocks import TransformerBlock
from .layernorm import LayerNorm

class Encoder(torch.nn.Module):
    """ Layers of encoder transformer blocks plus an final layernorm.

    Args:
        num_layers (int): number of layers.
        dim_model (int): main dimension of modules in transformer blocks.
        dim_ff (int): dim_ff used in :py:class:`model_center.layer.FeedForward`.
        num_heads (int): num_heads used in :py:class:`model_center.layer.Attention`.
        dim_head (int): dim_head used in :py:class:`model_center.layer.Attention`.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        att_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.Attention`. Defaults to 0.0.
        att_init_std (float, optional): init_std used in :py:class:`model_center.layer.Attention`. Defaults to 0.02.
        att_bias (bool, optional): bias used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        att_mask_value (float, optional): mask_value used in in :py:class:`model_center.layer.Attention`. Defaults to float("-inf").
        ffn_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.0.
        ffn_init_std (float, optional): init_std used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.02.
        ffn_bias (bool, optional): bias used in :py:class:`model_center.layer.FeedForward`. Defaults to False.
        ffn_activate_fn (str, optional): activate_fn used in :py:class:`model_center.layer.FeedForward`. Defaults to "gated_gelu".
        pos_bias_type (str, optional): pos_bias_type used in :py:class:`model_center.layer.Attention`. Defaults to "none".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        attn_scale (bool, optional): attn_scale used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        dropout_p (float, optional): Defaults to 0.
    """
    def __init__(self, 
            num_layers : int,
            dim_model : int, 
            dim_ff : int,
            num_heads : int,
            dim_head : int,
            dtype : torch.dtype = torch.half,
            int8 : bool = False, 
            norm_init_var : float = 1.0,
            norm_bias : bool = False,
            norm_eps : float = 1e-5, 
            att_init_mean : float = 0.0, 
            att_init_std : float = 0.02,
            att_bias : bool = False,
            att_mask_value : float = float("-inf"),
            ffn_init_mean : float = 0.0, 
            ffn_init_std : float = 0.02,
            ffn_bias : bool = False,
            ffn_activate_fn : str = "gated_gelu",
            pos_bias_type : str = "none",
            post_layer_norm : bool = False,
            length_scale : bool = False,
            attn_scale : bool = False,
            dropout_p : float = 0,
            parallel_ffn : bool = False,
            sparse_attention : bool = False,
            attention_window : int = 512,
            mask_modules : Optional[List[Tuple[int, int]]] = None,
        ):

        super().__init__()
        
        self.num_layers = num_layers

        if mask_modules is not None:
            assert len(mask_modules) == num_layers, "The total number of masks should equal to num_layers"
            for mask_module in mask_modules:
                assert len(mask_module) == 2, "For encoder, each mask should be (mask_att, mask_ffn)"
        else:
            mask_modules = [(False, False)] * num_layers

        self.layers = bmt.TransformerBlockList([
                bmt.CheckpointBlock(
                    TransformerBlock(
                        dim_model = dim_model, 
                        dim_ff = dim_ff,
                        num_heads = num_heads,
                        dim_head = dim_head,
                        is_decoder = False,
                        dtype = dtype, 
                        int8 = int8,
                        norm_eps = norm_eps, 
                        norm_init_var = norm_init_var,
                        norm_bias = norm_bias,
                        att_init_mean = att_init_mean, 
                        att_init_std = att_init_std,
                        att_bias = att_bias,
                        att_mask_value = att_mask_value,
                        ffn_init_mean = ffn_init_mean, 
                        ffn_init_std = ffn_init_std,
                        ffn_bias = ffn_bias,
                        ffn_activate_fn = ffn_activate_fn,
                        pos_bias_type = pos_bias_type,
                        post_layer_norm = post_layer_norm,
                        length_scale = length_scale,
                        attn_scale = attn_scale,
                        dropout_p = dropout_p,
                        parallel_ffn = parallel_ffn,
                        sparse_attention = sparse_attention,
                        attention_window = attention_window,
                        mask_att = mask_modules[ith][0],
                        mask_ffn = mask_modules[ith][1],
                    )
                )
                for ith in range(num_layers)
        ])

        self.output_layernorm = LayerNorm(
                    dim_norm = dim_model, 
                    bias = norm_bias, 
                    dtype = dtype,
                    eps = norm_eps,
                    init_var = norm_init_var)

    def forward(self, hidden_states : torch.Tensor,
                      attention_mask : torch.Tensor,
                      position_bias : torch.Tensor = None,
                      use_cache : bool = False,
                      past_key_values = None,
        ):
        """
        Args:
            hidden-states (:obj:`torch.Tensor` of shape ``(batch, seq_enc, dim_model)``): Input of encoder, might be the embedding of a batch of sequences. 
            attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_enc, seq_enc)``): Avoid invalid areas to participate in the calculation 
            position_bias(:obj:`torch.Tensor` of shape ``(num_heads, seq_enc, seq_enc)``) Provides position information to attention mechanism.  

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_enc, dim_model)``: The encoder output. 

        """
        if not use_cache:
            hidden_states = self.layers(hidden_states, attention_mask, position_bias, None, None, None)
            hidden_states = self.output_layernorm(hidden_states)
            return hidden_states
        else:
            with torch.no_grad():
                current_key_values = []
                for i, module in enumerate(self.layers):
                    hidden_states  = module(hidden_states, attention_mask, position_bias, 
                                            None, None, None, 
                                            past_key_value = past_key_values[i] if past_key_values else None, 
                                            use_cache = use_cache)
                    current_key_values.append(hidden_states[1])
                    hidden_states = hidden_states[0]
                hidden_states = self.output_layernorm(hidden_states)
                return hidden_states, current_key_values

class Decoder(torch.nn.Module):
    """ Layers of decoder transformer blocks plus an final layernorm.

    Args:
        num_layers (int): number of layers.
        dim_model (int): main dimension of modules in transformer blocks.
        dim_ff (int): dim_ff used in :py:class:`model_center.layer.FeedForward`.
        num_heads (int): num_heads used in :py:class:`model_center.layer.Attention`.
        dim_head (int): dim_head used in :py:class:`model_center.layer.Attention`.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        att_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.Attention`. Defaults to 0.0.
        att_init_std (float, optional): init_std used in :py:class:`model_center.layer.Attention`. Defaults to 0.02.
        att_bias (bool, optional): bias used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        att_mask_value (float, optional): mask_value used in in :py:class:`model_center.layer.Attention`. Defaults to float("-inf").
        ffn_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.0.
        ffn_init_std (float, optional): init_std used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.02.
        ffn_bias (bool, optional): bias used in :py:class:`model_center.layer.FeedForward`. Defaults to False.
        ffn_activate_fn (str, optional): activate_fn used in :py:class:`model_center.layer.FeedForward`. Defaults to "gated_gelu".
        pos_bias_type (str, optional): pos_bias_type used in :py:class:`model_center.layer.Attention`. Defaults to "none".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        attn_scale (bool, optional): attn_scale used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        dropout_p (float, optional): Defaults to 0.
    """
    def __init__(self, 
            num_layers : int,
            dim_model : int, 
            dim_ff : int,
            num_heads : int,
            dim_head : int,
            dtype : torch.dtype = torch.half,
            int8 : bool = False, 
            norm_init_var : float = 1.0,
            norm_bias : bool = False,
            norm_eps : float = 1e-5, 
            att_init_mean : float = 0.0, 
            att_init_std : float = 0.02,
            att_bias : bool = False,
            att_mask_value : float = float("-inf"),
            ffn_init_mean : float = 0.0, 
            ffn_init_std : float = 0.02,
            ffn_bias : bool = False,
            ffn_activate_fn : str = "gated_gelu",
            pos_bias_type : str = "none",
            length_scale : bool = False,
            attn_scale : bool = False,
            dropout_p : float = 0,
            parallel_ffn : bool = False,
            mask_modules : Optional[List[Tuple[int, int, int]]] = None,
        ):

        super().__init__()
        
        self.num_layers = num_layers

        if mask_modules is not None:
            assert len(mask_modules) == num_layers, "The total number of masks should equal to num_layers"
            for mask_module in mask_modules:
                assert len(mask_module) == 3, "For decoder, each mask should be (mask_att, mask_cross, mask_ffn)"
        else:
            mask_modules = [(False, False, False)] * num_layers
        
        self.layers = bmt.TransformerBlockList([
                bmt.CheckpointBlock(
                    TransformerBlock(
                        dim_model = dim_model, 
                        dim_ff = dim_ff,
                        num_heads = num_heads,
                        dim_head = dim_head,
                        is_decoder = True,
                        dtype = dtype, 
                        int8 = int8,
                        norm_init_var = norm_init_var,
                        norm_bias = norm_bias,
                        norm_eps = norm_eps, 
                        att_init_mean = att_init_mean, 
                        att_init_std = att_init_std,
                        att_bias = att_bias,
                        att_mask_value = att_mask_value,
                        ffn_init_mean = ffn_init_mean, 
                        ffn_init_std = ffn_init_std,
                        ffn_bias = ffn_bias,
                        ffn_activate_fn = ffn_activate_fn,
                        pos_bias_type = pos_bias_type,
                        length_scale = length_scale,
                        attn_scale = attn_scale,
                        dropout_p = dropout_p,
                        parallel_ffn = parallel_ffn,
                        mask_att = mask_modules[ith][0],
                        mask_cross = mask_modules[ith][1],
                        mask_ffn = mask_modules[ith][2],
                    )
                )
                for ith in range(num_layers)
        ])

        self.output_layernorm = LayerNorm(
                    dim_norm = dim_model, 
                    bias = norm_bias, 
                    dtype = dtype,
                    eps = norm_eps, 
                    init_var = norm_init_var)

    def forward(self, hidden_states : torch.Tensor,
                      attention_mask : torch.Tensor,
                      position_bias : torch.Tensor,
                      cross_hidden_states = None,
                      cross_attention_mask = None,
                      cross_position_bias = None,
                      use_cache : bool = False,
                      past_key_values = None,
        ):
        """
        Args:
            hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_dec, dim_model)``): Input of decoder, Can be the embedding of a batch of sequences. 
            attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_dec, seq_dec)``): Avoid invalid areas to participate in the calculation. 
            position_bias(:obj:`torch.Tensor` of shape ``(num_heads, seq_dec, seq_dec)``) Provides position information to attention mechanism. 
            cross_hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_enc, dim_model)``): Input of decoder, Can be the output of encoder. 
            cross_attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_dec, seq_enc)``): Avoid invalid areas to participate in the calculation when the output of encoder participates in the calculation. 
            cross_position_bias(:obj:`torch.Tensor` of shape ``(num_heads, seq_dec, seq_enc)``) Provides position information to attention mechanism when the output of encoder participates in the calculation.  

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_dec, dim_model)``: The decoder output. 

        """
        if not use_cache:
            hidden_states = self.layers(hidden_states, attention_mask, position_bias,
                                        cross_hidden_states, cross_attention_mask, cross_position_bias)
            hidden_states = self.output_layernorm(hidden_states)
            return hidden_states
        else:
            with torch.no_grad():
                current_key_values = []
                for i, module in enumerate(self.layers):
                    hidden_states  = module(hidden_states, attention_mask, position_bias, 
                                            cross_hidden_states, cross_attention_mask, cross_position_bias,
                                            past_key_value = past_key_values[i] if past_key_values else None, 
                                            use_cache = use_cache)
                    current_key_values.append(hidden_states[1])
                    hidden_states = hidden_states[0]
                hidden_states = self.output_layernorm(hidden_states)
                return hidden_states, current_key_values



# coding=utf-8
# Copyright 2020 The OpenBMB team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import torch
import csv
import numpy as np

class LCQMC_Dataset(torch.utils.data.Dataset):
    def __init__(self, path, split, rank, world_size, tokenizer, max_length) -> None:
        self.data = []

        path = f"{path}/LCQMC/{split}.tsv"
        with open(path, encoding='utf8') as fin:
            reader = list(csv.reader(fin, delimiter='\t'))[1:]
            for i, row in enumerate(reader):
                text_a, text_b, label = row
                lef_tokens = [1] + tokenizer.encode(f'"{text_a}"与"{text_b}"的关系是:')
                rig_tokens = tokenizer.encode("。")

                input_tokens, input_length, context, input_span = self.make_input(lef_tokens, rig_tokens, 1, max_length)

                index = torch.zeros((max_length,), dtype=torch.int32)
                index[len(lef_tokens) - 1] = 1

                target = torch.tensor(int(label), dtype=torch.long)

                self.data.append({
                    "input_tokens": input_tokens.cuda(),
                    "input_length": input_length.cuda(),
                    "input_context": context.cuda(),
                    "input_span": input_span.cuda(),
                    "targets": target.cuda(),
                    "index": index.cuda(),
                })

    def make_input(self, lef_tokens, rig_tokens, spans, max_length):
        input = lef_tokens + [0 for i in range(spans)] + rig_tokens
        length = len(input)

        assert length < max_length # TODO

        input_tokens = torch.zeros((max_length,), dtype=torch.int32)
        input_tokens[:length] = torch.tensor(input).int()

        input_length = torch.tensor(length, dtype=torch.int32)

        context = np.arange(max_length)
        context = (context < len(lef_tokens)) | (context >= len(lef_tokens) + spans)
        context = torch.from_numpy(context).bool()

        input_span = torch.zeros((max_length,), dtype=torch.int32)

        return input_tokens, input_length, context, input_span

    def __len__(self):
        return len(self.data)
    
    def __getitem__(self, idx):
        return self.data[idx]

    @classmethod
    def get_verbalizer(cls, tokenizer):
        return [15682, 16357] # 有关，无关 # TODO change to tokenizer.encode(xxx)


# coding=utf-8
# Copyright 2020 The OpenBMB team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import struct
import shutil

from itertools import accumulate

import numpy as np
import torch
import bmtrain as bmt

dtypes = {
    1: np.uint8,
    2: np.int8,
    3: np.int16,
    4: np.int32,
    5: np.int64,
    6: np.float32,
    7: np.double,
    8: np.uint16
}


def code(dtype):
    for k in dtypes.keys():
        if dtypes[k] == dtype:
            return k
    raise ValueError(dtype)


def index_file_path(prefix_path):
    return prefix_path + '.idx'


def data_file_path(prefix_path):
    return prefix_path + '.bin'


class DistributedMMapIndexedDataset(torch.utils.data.Dataset):
    class Index(object):
        _HDR_MAGIC = b'MMIDIDX\x00\x00'
        def __init__(self, path):
            with open(path, 'rb') as stream:
                magic_test = stream.read(9)
                assert self._HDR_MAGIC == magic_test, (
                    'Index file doesn\'t match expected format. '
                    'Make sure that --dataset-impl is configured properly.'
                )
                version = struct.unpack('<Q', stream.read(8))
                assert (1,) == version

                dtype_code, = struct.unpack('<B', stream.read(1))
                self._dtype = dtypes[dtype_code]
                self._dtype_size = self._dtype().itemsize

                self._len = struct.unpack('<Q', stream.read(8))[0]
                self._doc_count = struct.unpack('<Q', stream.read(8))[0]
                offset = stream.tell()

            self._bin_buffer_mmap = np.memmap(path, mode='r', order='C')
            self._bin_buffer = memoryview(self._bin_buffer_mmap)
            self._sizes = np.frombuffer(
                self._bin_buffer,
                dtype=np.int32,
                count=self._len,
                offset=offset)
            self._pointers = np.frombuffer(self._bin_buffer, dtype=np.int64, count=self._len,
                                           offset=offset + self._sizes.nbytes)
            self._doc_idx = np.frombuffer(self._bin_buffer, dtype=np.int64, count=self._doc_count,
                                          offset=offset + self._sizes.nbytes + self._pointers.nbytes)

        def __del__(self):
            self._bin_buffer_mmap._mmap.close()
            del self._bin_buffer_mmap

        @property
        def dtype(self):
            return self._dtype

        @property
        def sizes(self):
            return self._sizes

        @property
        def doc_idx(self):
            return self._doc_idx

        def __getitem__(self, i):
            return self._pointers[i], self._sizes[i]

        def __len__(self):
            return self._len

    def __init__(self, path, name, rank_number, rank_total, cache = None):
        
        super().__init__()

        self._path = path
        self._name = name
        self._state = 0
        if cache is not None:
            self._cache = cache
            os.makedirs(self._cache, exist_ok=True)
        else:
            self._cache = None
        self._rank_total = rank_total
        self._rank_number = rank_number
        self._index = None
        self._bin_buffer = None
        self._bin_buffer_mmap = None
        self.history = {-1:0}

        self._do_init(self._path, self._name, self._cache, self._state)

    def __getstate__(self):
        return self._path + self._name + "_%d"%(self._state)

    def __setstate__(self, state):
        self._state = state
        self._do_init(self._path, self._name, self._cache, self._state)

    def _do_init(self, path, name, cache, state):
        if self._bin_buffer_mmap is not None:
            self._bin_buffer_mmap._mmap.close()
            del self._bin_buffer_mmap
        if self._index is not None:
            del self._index

        self._state = state

        source_file = path + name + "_%d"%(self._state)
        self._index = self.Index(index_file_path(source_file))
        self._bin_buffer_mmap = np.memmap(data_file_path(source_file), mode='r', order='C')
        self._bin_buffer = memoryview(self._bin_buffer_mmap)

        self.history[state] = self.history[state - 1] + len(self._index) // self._rank_total
        self.start = (len(self._index) // self._rank_total) * self._rank_number
        print (self.history[state],"======================", self.start, self.history[self._state - 1])

    def __del__(self):
        if self._bin_buffer_mmap is not None:
            self._bin_buffer_mmap._mmap.close()
            del self._bin_buffer_mmap
        if self._index is not None:
            del self._index

    def _next_file(self):
        self._state += 1
        bmt.print_rank(f"next_file: {self._state}")
        self._do_init(self._path, self._name, self._cache, self._state)
    
    def __relative_idx(self, idx):
        res = self.start + idx - self.history[self._state - 1]
        return res

    def __slice_item(self, start, stop):
        ptr = self._index._pointers[self.__relative_idx(start)]
        sizes = self._index._sizes[self.__relative_idx(start):self.__relative_idx(stop)]
        offsets = list(accumulate(sizes))
        np_array = np.frombuffer(self._bin_buffer, dtype=self._index.dtype, count=sum(sizes), offset=ptr)
        return np.split(np_array, offsets[:-1])

    def __getitem__(self, idx):
        if isinstance(idx, int):
            while idx >= self.history[self._state]:
                self._next_file()
            ptr, size = self._index[self.__relative_idx(idx)]
            return np.frombuffer(self._bin_buffer, dtype=self._index.dtype, count=size, offset=ptr)
        elif isinstance(idx, slice):
            start, stop, step = idx.indices(2147483647)
            assert step == 1 or step is None, "Slices into indexed_dataset must be contiguous"
            if stop >= self.history[self._state]:
                res_1 = self.__slice_item(start, self.history[self._state]) 
                self._next_file()
                res_2 = self.__slice_item(self.history[self._state - 1], stop)
                return res_1 + res_2
            else:
                return self.__slice_item(start, stop)

    @property
    def sizes(self):
        return self._index.sizes
        
    def exists(path):
        return (
            os.path.exists(index_file_path(path)) and os.path.exists(data_file_path(path))
        )



# coding=utf-8
# Copyright 2020 The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import requests
import tqdm
import bmtrain as bmt

file_names = {
    'config': ['config.json'],
    'model': ['pytorch_model.pt'],
    'tokenizer': ['vocab.json', 'vocab.txt', 'merges.txt', 'tokenizer.json', 'added_tokens.json', 'special_tokens_map.json', 'tokenizer_config.json', 'spiece.model', 'vocab.model'],
}

def download(path, url):
    req = requests.get(url, stream=True)
    try:
        os.makedirs(os.path.dirname(path), exist_ok=True)
        file = open(path, "wb")
        req.raise_for_status()
        print(f"download from web, cache will be save to: {path}")
        content_length = req.headers.get("Content-Length")
        total = int(content_length) if content_length is not None else None
        progress = tqdm.tqdm(
            unit="B",
            unit_scale=True,
            unit_divisor=1024,
            total=total,
            desc="Downloading",
        )
        for chunk in req.iter_content(chunk_size=1024):
            if chunk:
                progress.update(len(chunk))
                file.write(chunk)
        progress.close()
        file.close()
    except:
        file.close()
        os.remove(path)

def check_web_and_convert_path(path, load_type): # TODO add hash
    if os.path.isdir(path):
        try:
            bmt.print_rank(f"load from local file: {path}")
        except:
            pass
        return path
    else:
        if bmt.rank() == 0:
            url = f"https://openbmb.oss-cn-hongkong.aliyuncs.com/model_center/{path}"
            try:
                requests.get(f'{url}/config.json', stream=True).raise_for_status() # use config.json to check if identifier is valid
            except:
                raise ValueError(f"'{path}' is not a valid model identifier")
            cache_path = os.path.expanduser(f"~/.cache/model_center/{path}")
            for name in file_names[load_type]:
                p = os.path.join(cache_path, name)
                if os.path.exists(p):
                    bmt.print_rank(f"load from cache: {p}")
                else:
                    if bmt.rank() == 0:
                        download(p, f"{url}/{name}")
        else:
            cache_path = os.path.expanduser(f"~/.cache/model_center/{path}")
        try:
            bmt.synchronize()
        except:
            pass
        return cache_path
        



# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
import bmtrain as bmt
import math
import torch.nn.functional as F

class Linear(bmt.DistributedModule):
    r"""A fully connected layer, which performs :math:`\pmb{y} = \mathbf{W} \pmb{x} + \pmb{b}`

    Args:
        dim_in (int): input dimension of :math:`\pmb{x}`
        dim_out (int): output dimension of :math:`\pmb{y}`
        dtype (optional): Defaults to torch.half.
        init_mean (float, optional): mean of :math:`\mathbf{W}\sim\mathcal{N}(\text{mean}, \text{std}^2)`. Defaults to 0.
        init_std (float, optional): std of :math:`\mathbf{W}\sim\mathcal{N}(\text{mean}, \text{std}^2)`. Defaults to 1.
        bias (bool, optional): whether to add bias term :math:`\pmb{b}`. Defaults to False.
    """
    def __init__(self,
                 dim_in : int,
                 dim_out : int,
                 length_scale : bool = False,
                 length_scale_before : bool = False,
                 dtype = torch.half,
                 int8 : bool = False,
                 init_mean : float = 0.0,
                 init_std : float = 1,
                 bias : bool = False,
                ):
        super().__init__()
        self.dim_in = self.in_features = dim_in
        self.dim_out = self.out_features = dim_out
        self.weight = bmt.DistributedParameter(
            torch.empty((dim_out, dim_in), dtype=dtype),
            init_method=bmt.ParameterInitializer(torch.nn.init.normal_, mean=init_mean, std=init_std)
        )
        self.bias = bmt.DistributedParameter(
            torch.empty((dim_out,), dtype=dtype),
            init_method=bmt.ParameterInitializer(torch.nn.init.zeros_)
        ) if bias else None
        self.length_scale = length_scale
        self.length_scale_before = length_scale_before
        self.int8 = int8

    def forward(self, x : torch.Tensor):
        """ 
        Args:
            x (:obj:`torch.Tensor` of shape ``(batch, seq_len, dim_in)``): The input of linear layer

        Returns:
            :obj:`torch.Tensor` of shape ``(batch, seq_len, dim_out)``: The output of the linear transform y.

        """
        if self.length_scale and self.length_scale_before:
            x = x / math.sqrt(self.dim_in)
        x = F.linear(x, self.weight)
        if self.length_scale and not self.length_scale_before:
            x = x / math.sqrt(self.dim_in)
        if self.bias is not None:
            x = x + self.bias
        return x



# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
import torch.nn as nn
from typing import Optional, List
from ..layer import Encoder, Embedding, Linear
from .basemodel import BaseModel, BaseModelOutput
from .config import OPTConfig

class OPT(BaseModel):

    _CONFIG_TYPE = OPTConfig

    def __init__(self, config: OPTConfig):
        
        super().__init__()
        # Model Config
        self.config = config
        # Embedding Layer
        self.input_embedding = Embedding(
            vocab_size = config.vocab_size,
            embedding_size = config.dim_model,
            length_scale = config.length_scale,
            dtype = config.dtype,
            int8 = config.int8,
            init_mean = config.emb_init_mean,
            init_std = config.emb_init_std,
            padding_idx = config.pad_token_id,
        )
        self.position_embedding = Embedding(
            vocab_size = config.position_size+2,
            embedding_size = config.dim_model,
            length_scale = config.length_scale,
            dtype = config.dtype,
            int8 = config.int8,
            init_mean = config.emb_init_mean,
            init_std = config.emb_init_std,
        )
        self.embed_dropout = nn.Dropout(config.dropout_p)
        # OPT Model
        self.decoder = Encoder(
            num_layers = config.num_layers,
            dim_model = config.dim_model,
            dim_ff = config.dim_ff,
            num_heads = config.num_heads,
            dim_head = config.dim_head,
            dtype = config.dtype,
            int8 = config.int8,
            norm_eps = config.norm_eps,
            norm_init_var = config.norm_init_var,
            norm_bias = config.norm_bias,
            att_init_mean = config.att_init_mean,
            att_init_std = config.att_init_std,
            att_bias = config.att_bias,
            att_mask_value = float(config.att_mask_value),
            pos_bias_type = config.pos_bias_type,
            ffn_init_mean = config.ffn_init_mean, 
            ffn_init_std = config.ffn_init_std,
            ffn_bias = config.ffn_bias,
            ffn_activate_fn = config.ffn_activate_fn,
            length_scale = config.length_scale,
            attn_scale = config.attn_scale,
            dropout_p = config.dropout_p,
        )
        # Output Layer
        if config.cls_head:
            self.cls_projection = Linear(
                dim_out = config.cls_head,
                dim_in = config.dim_model,
                length_scale = config.length_scale,
                dtype = config.dtype,
                int8 = config.int8,
                init_mean = config.proj_init_mean,
                init_std = config.proj_init_std,
                bias = config.proj_bias,
            )

    def forward(self,
                input_ids: Optional[torch.Tensor] = None,
                length: Optional[torch.Tensor] = None,
                attention_mask: Optional[torch.Tensor] = None,
                token_type_ids: Optional[torch.Tensor] = None,
                position_ids: Optional[torch.Tensor] = None,
                head_mask: Optional[torch.Tensor] = None,
                inputs_embeds: Optional[torch.FloatTensor] = None,
                use_cache: Optional[bool] = False,
                past_key_values: Optional[List[torch.FloatTensor]] = None,
                output_logits: Optional[bool] = False,
                output_attentions: Optional[bool] = False,
                output_hidden_states: Optional[bool] = False,
                return_dict: Optional[bool] = True,
        ):
        """ This model inherits from BaseModel. This model is also a PyTorch torch.nn.Module subclass.
            You can use it as a regular PyTorch Module. You can also select the data and data type that 
            you want the model to return through changing the value of `output_logits`, 
            `output_pooler_output`, `output_attentions`, `output_hidden_states` and `return_dict`.

        Args:
            input_ids (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of input sequence tokens in the vocabulary.
            length (`torch.LongTensor` of shape `(batch_size)`, *optional*):
                Length of input sequence before padding.  
            attention_mask (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Mask to avoid performing attention on padding token indices. The values are selected in `[0, 1]`:
                - 1 for tokens that are **not masked**,
                - 0 for tokens that are **masked**.
                At least one of `length` and `attention_mask` must be given.
            token_type_ids (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Segment token indices to indicate first and second portions of the inputs. The values are selected in `[0, 1]`:
                - 0 corresponds to a *sentence A* token,
                - 1 corresponds to a *sentence B* token.
                Unused now.
            position_ids (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of positions of each input sequence tokens in the position embeddings. The values are selected in the range `[0,
                config.position_size - 1]`.
            head_mask (`torch.LongTensor` of shape `(num_heads,)` or `(num_layers, num_heads)`, *optional*):
                Mask to nullify selected heads of the self-attention modules. The values are selected in `[0, 1]`:
                - 1 indicates the head is **not masked**,
                - 0 indicates the head is **masked**.
                Unused now.
            inputs_embeds (`torch.FloatTensor` of shape `({0}, hidden_size)`, *optional*):
                Optionally, instead of passing `input_ids` you can choose to directly pass an embedded representation. This
                is useful if you want to convert `input_ids` indices into associated vectors rather than the model's internal 
                token vectors.
            use_cache (`bool`, *optional*):
                If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding (see `past_key_values`).
            past_key_values (`tuple(tuple(torch.FloatTensor))` of length `config.num_layers` with each tuple having 4 tensors of shape `(batch_size, num_heads, sequence_length - 1, dim_model)`):
                Contains precomputed key and value hidden states of the attention blocks. Can be used to speed up decoding.
            output_logits (`bool`, *optional*):
                Whether or not to return the prediction score for each token in vocabulary (before softmax).
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers. 
                Unused now.
            output_hidden_states (`bool`, *optional*):
                Whether or not to return the hidden states of all layers.
                Unused now.
            return_dict (`bool`, *optional*):
                Whether or not to return a [`BaseModelOutput`] instead of a plain tuple.

        Return:
            BaseModelOutput or tuple: The OPT output. 
            Depended on the value of `output_logits`, `output_attentions`, `output_hidden_states` and `return_dict`.
        """

        # encode the input into embeddings.
        with torch.no_grad():
            assert input_ids is not None or inputs_embeds is not None
            if input_ids is not None:
                batch = input_ids.size(0)
                input_length = input_ids.size(1)
                device = input_ids.device
            else:
                batch = inputs_embeds.size(0)
                input_length = inputs_embeds.size(1)
                device = inputs_embeds.device
            pkv_len = 0 if past_key_values is None else past_key_values[0][0].size(-2)
            seq_length = pkv_len + input_length

            if attention_mask is None and length is None:
                length = torch.ones((batch,), dtype=torch.int32, device=device) * seq_length
            attention_mask = attention_mask.to(torch.bool) if attention_mask is not None else \
                torch.arange(seq_length, device=device)[None, :].repeat(batch, 1) < length[:, None]
            if attention_mask.dim() == 2:
                if position_ids is None:
                    position_ids = (torch.cumsum(attention_mask, dim=1) * attention_mask).int() + 1
                directional_mask_2d = torch.arange(seq_length, device=device) <= torch.arange(seq_length, device=device).view(-1, 1)
                attention_mask = attention_mask.view(batch, 1, seq_length) & directional_mask_2d.view(1, seq_length, seq_length)
            attention_mask = attention_mask[:, -input_length:, :]

            position_ids = position_ids[:, -input_length:]

        if inputs_embeds is None:
            hidden_states = self.input_embedding(input_ids)
        else:
            hidden_states = inputs_embeds
        position_embeds = self.position_embedding(position_ids)

        hidden_states = hidden_states + position_embeds
        hidden_states = self.embed_dropout(hidden_states)

        # input the input embeddings into the OPT model
        current_key_values = None
        if use_cache:
            hidden_states, current_key_values = self.decoder(hidden_states, attention_mask, 
                                                             use_cache = use_cache, past_key_values = past_key_values)
        else:
            hidden_states = self.decoder(hidden_states, attention_mask)
        # use the hidden states of the last layer for sequential tasks, such as sequential labeling and language modeling.
        logits = None
        if output_logits:
            if self.config.cls_head:
                logits = self.cls_projection(hidden_states)
            else:
                logits = self.input_embedding.projection(hidden_states)

        # BaseModelOutput or tuple: The OPT output. 
        if not return_dict:
            return hidden_states, current_key_values, logits, None, None
        else:
            return BaseModelOutput(
                last_hidden_state = hidden_states,
                past_key_values = current_key_values,
                logits = logits,
                hidden_states = None,
                attentions = None,
            )



# coding=utf-8
# Copyright 2020 The OpenBMB team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import struct
from itertools import accumulate

import numpy as np
import torch

dtypes = {
    1: np.uint8,
    2: np.int8,
    3: np.int16,
    4: np.int32,
    5: np.int64,
    6: np.float32,
    7: np.double,
    8: np.uint16
}


def code(dtype):
    for k in dtypes.keys():
        if dtypes[k] == dtype:
            return k
    raise ValueError(dtype)


def index_file_path(prefix_path):
    return prefix_path + '.idx'


def data_file_path(prefix_path):
    return prefix_path + '.bin'


class MMapIndexedDataset(torch.utils.data.Dataset):
    class Index(object):
        _HDR_MAGIC = b'MMIDIDX\x00\x00'
        def __init__(self, path):
            with open(path, 'rb') as stream:
                magic_test = stream.read(9)
                assert self._HDR_MAGIC == magic_test, (
                    'Index file doesn\'t match expected format. '
                    'Make sure that --dataset-impl is configured properly.'
                )
                version = struct.unpack('<Q', stream.read(8))
                assert (1,) == version

                dtype_code, = struct.unpack('<B', stream.read(1))
                self._dtype = dtypes[dtype_code]
                self._dtype_size = self._dtype().itemsize

                self._len = struct.unpack('<Q', stream.read(8))[0]
                self._doc_count = struct.unpack('<Q', stream.read(8))[0]
                offset = stream.tell()


            self._bin_buffer_mmap = np.memmap(path, mode='r', order='C')
            self._bin_buffer = memoryview(self._bin_buffer_mmap)
            self._sizes = np.frombuffer(
                self._bin_buffer,
                dtype=np.int32,
                count=self._len,
                offset=offset)
            self._pointers = np.frombuffer(self._bin_buffer, dtype=np.int64, count=self._len,
                                           offset=offset + self._sizes.nbytes)
            self._doc_idx = np.frombuffer(self._bin_buffer, dtype=np.int64, count=self._doc_count,
                                          offset=offset + self._sizes.nbytes + self._pointers.nbytes)

        def __del__(self):
            self._bin_buffer_mmap._mmap.close()
            del self._bin_buffer_mmap

        @property
        def dtype(self):
            return self._dtype

        @property
        def sizes(self):
            return self._sizes

        @property
        def doc_idx(self):
            return self._doc_idx

        def __getitem__(self, i):
            return self._pointers[i], self._sizes[i]

        def __len__(self):
            return self._len

    def __init__(self, path):
        super().__init__()

        self._path = None
        self._index = None
        self._bin_buffer = None

        self._do_init(path)

    def __getstate__(self):
        return self._path

    def __setstate__(self, state):
        self._do_init(state)

    def _do_init(self, path):
        self._path = path
        self._index = self.Index(index_file_path(self._path))

        self._bin_buffer_mmap = np.memmap(data_file_path(self._path), mode='r', order='C')
        self._bin_buffer = memoryview(self._bin_buffer_mmap)

    def __del__(self):
        self._bin_buffer_mmap._mmap.close()
        del self._bin_buffer_mmap
        del self._index

    def __len__(self):
        return len(self._index)

    def __getitem__(self, idx):
        if isinstance(idx, int):
            assert idx < len(self._index), "Index {} out of range: {}".format(idx, len(self._index))
            ptr, size = self._index[idx]
            np_array = np.frombuffer(self._bin_buffer, dtype=self._index.dtype,
                                     count=size, offset=ptr)
            return np_array
        elif isinstance(idx, slice):
            start, stop, step = idx.indices(len(self))
            if step != 1:
                raise ValueError("Slices into indexed_dataset must be contiguous")
            ptr = self._index._pointers[start]
            sizes = self._index._sizes[idx]
            offsets = list(accumulate(sizes))
            total_size = sum(sizes)
            np_array = np.frombuffer(self._bin_buffer, dtype=self._index.dtype,
                                     count=total_size, offset=ptr)
            sents = np.split(np_array, offsets[:-1])
            return sents

    @property
    def sizes(self):
        return self._index.sizes
        
    def exists(path):
        return (
            os.path.exists(index_file_path(path)) and os.path.exists(data_file_path(path))
        )



# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
from .config import Config

class CPM3Config(Config):

    def __init__(self, vocab_size = 30720,
                        dim_model = 4096,
                        num_heads = 64,
                        dim_head = 64,
                        dim_ff = 10240,
                        num_layers = 32,
                        dropout_p = 0.0,
                        emb_init_mean = 0.0,
                        emb_init_std = 1.0,
                        pos_bias_type = "relative",
                        position_bias_num_buckets = 512,
                        position_bias_max_distance = 2048,
                        pos_init_mean = 0.0,
                        pos_init_std = 1.0,
                        norm_init_var = 1.0,
                        norm_bias = False,
                        norm_eps = 1e-6,
                        att_init_mean = 0.0, 
                        att_init_std = 1.0,
                        att_bias = False,
                        att_mask_value = float("-inf"),
                        ffn_init_mean = 0.0, 
                        ffn_init_std = 1.0,
                        ffn_bias = False,
                        ffn_activate_fn = "gated_gelu",
                        proj_init_mean = 0.0,
                        proj_init_std = 1.0,
                        proj_bias = False,
                        length_scale = True,
                        attn_scale = True,
                        half = True, 
                        int8 = False,
                        tied = True,
                        prompt_types = 32,
                        prompt_length = 64, 
                        segment_types = 34,
                        max_exact_rate = 0.25,
                        max_distance_rate = 1.0,
                        absolute_inner_segment = True,
                        cls_head = None,
                        post_layer_norm=False,
        ):

        super().__init__()
        self.prompt_types = prompt_types
        self.prompt_length = prompt_length
        self.segment_types = segment_types
        self.dim_model = dim_model
        self.num_heads = num_heads
        self.dim_head = dim_head
        self.dim_ff = dim_ff
        self.num_layers = num_layers
        self.position_bias_num_buckets = position_bias_num_buckets
        self.position_bias_max_distance = position_bias_max_distance
        self.dropout_p = dropout_p
        self.norm_eps = norm_eps
        self.norm_init_var = norm_init_var
        self.emb_init_mean = emb_init_mean
        self.emb_init_std = emb_init_std
        self.att_init_mean = att_init_mean
        self.att_init_std = att_init_std
        self.ffn_init_mean = ffn_init_mean
        self.ffn_init_std = ffn_init_std
        self.length_scale = length_scale
        self.absolute_inner_segment = absolute_inner_segment
        self.max_distance_rate = max_distance_rate
        self.max_exact_rate = max_exact_rate
        self.int8 = int8
        self.tied = tied
        if half: 
            self.dtype = torch.half
        else:
            self.dtype = torch.float
        self.cls_head = cls_head
        self.vocab_size = vocab_size
        self.pos_bias_type = pos_bias_type
        self.pos_init_mean = pos_init_mean
        self.pos_init_std = pos_init_std
        self.norm_bias = norm_bias
        self.att_bias = att_bias
        self.att_mask_value = att_mask_value
        self.ffn_bias = ffn_bias
        self.ffn_activate_fn = ffn_activate_fn
        self.proj_init_mean = proj_init_mean
        self.proj_init_std = proj_init_std
        self.proj_bias = proj_bias
        self.attn_scale = attn_scale
        self.post_layer_norm = post_layer_norm


# coding=utf-8
# Copyright 2022 The OpenBMB team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
from typing import Optional, List
from ..layer import Encoder, Embedding, Linear, RelativePositionEmbedding
from .basemodel import BaseModel, BaseModelOutput
from .config import CPM1Config


class CPM1(BaseModel):

    _CONFIG_TYPE = CPM1Config

    def __init__(self, config: CPM1Config):

        super().__init__()
        # Model Config
        self.config = config
        # Embedding Layer
        self.input_embedding = Embedding(
            vocab_size = config.vocab_size, 
            embedding_size = config.dim_model,
            length_scale = config.length_scale,
            dtype = config.dtype,
            int8 = config.int8,
            init_mean = config.emb_init_mean,
            init_std = config.emb_init_std,
        )
        self.position_bias = RelativePositionEmbedding(
            num_heads = config.num_heads, 
            num_buckets = config.position_bias_num_buckets, 
            max_distance = config.position_bias_max_distance, 
            bidirectional = True,
            dtype = config.dtype,
            init_mean = config.pos_init_mean,
            init_std = config.pos_init_std,
        )
        # CPM-1 Model
        self.encoder = Encoder(
            num_layers = config.num_layers,
            dim_model = config.dim_model, 
            dim_ff = config.dim_ff,
            num_heads = config.num_heads,
            dim_head = config.dim_head,
            dtype = config.dtype, 
            int8 = config.int8,
            norm_eps = config.norm_eps, 
            norm_init_var = config.norm_init_var,
            norm_bias = config.norm_bias,
            att_init_mean = config.att_init_mean, 
            att_init_std = config.att_init_std,
            att_bias = config.att_bias,
            att_mask_value = float(config.att_mask_value),
            pos_bias_type = config.pos_bias_type,
            ffn_init_mean = config.ffn_init_mean, 
            ffn_init_std = config.ffn_init_std,
            ffn_bias = config.ffn_bias,
            ffn_activate_fn = config.ffn_activate_fn,
            length_scale = config.length_scale,
            attn_scale = config.attn_scale,
            dropout_p = config.dropout_p,        
        )
        # Output Layer
        if config.cls_head:
            self.cls_projection = Linear(
                dim_out = config.cls_head,
                dim_in = config.dim_model,
                length_scale = config.length_scale,
                dtype = config.dtype,
                int8 = config.int8,
                init_mean = config.proj_init_mean,
                init_std = config.proj_init_std,
                bias = config.proj_bias,
            )
        if not config.tied:
            self.output_projection = Linear(
                dim_out = config.vocab_size,
                dim_in = config.dim_model,
                length_scale = config.length_scale,
                dtype = config.dtype,
                int8 = config.int8,
                init_mean = config.proj_init_mean,
                init_std = config.proj_init_std,
                bias = config.proj_bias,
            )

    def forward(self, 
                input_ids: Optional[torch.Tensor] = None,
                length: Optional[torch.Tensor] = None,
                attention_mask: Optional[torch.Tensor] = None,
                token_type_ids: Optional[torch.Tensor] = None,
                position_ids: Optional[torch.Tensor] = None,
                context: Optional[torch.Tensor] = None,
                span: Optional[torch.Tensor] = None,
                head_mask: Optional[torch.Tensor] = None,
                inputs_embeds: Optional[torch.FloatTensor] = None,
                use_cache: Optional[bool] = False,
                past_key_values: Optional[List[torch.FloatTensor]] = None,
                output_logits: Optional[bool] = False,
                output_attentions: Optional[bool] = False,
                output_hidden_states: Optional[bool] = False,
                return_dict: Optional[bool] = True,
        ):
        """ This model inherits from BaseModel. This model is also a PyTorch torch.nn.Module subclass.
            You can use it as a regular PyTorch Module. You can also select the data and data type that 
            you want the model to return through changing the value of `output_logits`, 
            `output_pooler_output`, `output_attentions`, `output_hidden_states` and `return_dict`.

        Args:
            input_ids (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of input sequence tokens in the vocabulary.
            length (`torch.LongTensor` of shape `(batch_size)`, *optional*):
                Length of input sequence before padding.
            attention_mask (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Mask to avoid performing attention on padding token indices. The values are selected in `[0, 1]`:
                - 1 for tokens that are **not masked**,
                - 0 for tokens that are **masked**.
                At least one of `length` and `attention_mask` must be given.
            token_type_ids (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Segment token indices to indicate first and second portions of the inputs. 
            position_ids (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of positions of each input sequence tokens in the position embeddings. 
            context (`torch.BoolTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Value to show whether tokens are contexts or not.
                - Ture for tokens that are contexts
                - False for tokens that are not contexts
            span (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of input sequence spans. The attention between the tokens in different spans is set to 
                negative infinity. This can bundle multiple inputs for processing.
            head_mask (`torch.LongTensor` of shape `(num_heads,)` or `(num_layers, num_heads)`, *optional*):
                Mask to nullify selected heads of the self-attention modules. The values are selected in `[0, 1]`:
                - 1 indicates the head is **not masked**,
                - 0 indicates the head is **masked**.
                Unused now.
            inputs_embeds (`torch.FloatTensor` of shape `({0}, hidden_size)`, *optional*):
                Optionally, instead of passing `input_ids` you can choose to directly pass an embedded representation. This
                is useful if you want to convert `input_ids` indices into associated vectors rather than the model's internal 
                token vectors.
            use_cache (`bool`, *optional*):
                If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding (see `past_key_values`).
            past_key_values (`tuple(tuple(torch.FloatTensor))` of length `config.num_layers` with each tuple having 4 tensors of shape `(batch_size, num_heads, sequence_length - 1, dim_model)`):
                Contains precomputed key and value hidden states of the attention blocks. Can be used to speed up decoding.
            output_logits (`bool`, *optional*):
                Whether or not to return the prediction score for each token in vocabulary (before softmax).
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers. 
                Unused now.
            output_hidden_states (`bool`, *optional*):
                Whether or not to return the hidden states of all layers.
                Unused now.
            return_dict (`bool`, *optional*):
                Whether or not to return a [`BaseModelOutput`] instead of a plain tuple.

        Return:
            BaseModelOutput or tuple: The CPM-1 output. 
            Depended on the value of `output_logits`, `output_attentions`, `output_hidden_states` and `return_dict`.
        """

        # encode the input into embeddings.
        with torch.no_grad():
            assert input_ids is not None or inputs_embeds is not None
            if input_ids is not None:
                batch = input_ids.size(0)
                input_length = input_ids.size(1)
                device = input_ids.device
            else:
                batch = inputs_embeds.size(0)
                input_length = inputs_embeds.size(1)
                device = inputs_embeds.device
            pkv_len = 0 if past_key_values is None else past_key_values[0][0].size(-2)
            seq_length = pkv_len + input_length

            attention_mask = attention_mask.to(torch.bool) if attention_mask is not None else \
                torch.arange(seq_length, device=device)[None, :].repeat(batch, 1) < length[:, None]
            if attention_mask.dim() == 2:
                directional_mask_2d = torch.arange(seq_length, device=device) <= torch.arange(seq_length, device=device).view(-1, 1)
                if context is None:
                    attention_mask = attention_mask.view(batch, 1, seq_length) & directional_mask_2d.view(1, seq_length, seq_length)
                else:
                    attention_mask = attention_mask.view(batch, seq_length, 1) & attention_mask.view(batch, 1, seq_length) & \
                        (context[:, None, :] | (context[:, :, None].logical_not() & directional_mask_2d.view(1, seq_length, seq_length)))
            if span is not None:
                attention_mask = (span[:, None, :] == span[:, :, None]) & attention_mask
            attention_mask = attention_mask[:, -input_length:, :]

        position_bias = self.position_bias(position_ids, position_ids) if position_ids is not None else \
            self.position_bias(seq_length, seq_length)
        position_bias = position_bias[:, -input_length:, :]
        hidden_states = self.input_embedding(input_ids) if inputs_embeds is None else inputs_embeds

        # input the input embeddings into the CPM-1 model
        current_key_values = None
        if use_cache:
            hidden_states, current_key_values = self.encoder(hidden_states, attention_mask, position_bias,
                                                             use_cache = use_cache, past_key_values = past_key_values)
        else:
            hidden_states = self.encoder(hidden_states, attention_mask, position_bias)

        # use the hidden states of the last layer for sequential tasks, such as sequential labeling and language modeling.
        logits = None
        if output_logits:
            if self.config.cls_head:
                logits = self.cls_projection(hidden_states)
            elif self.config.tied:
                logits = self.input_embedding.projection(hidden_states)
            elif not self.config.tied:
                logits = self.output_projection(hidden_states)

        # BaseModelOutput or tuple: The CPM-1 output. 
        if not return_dict:
            return hidden_states, current_key_values, logits, None, None
        else:
            return BaseModelOutput(
                last_hidden_state = hidden_states,
                past_key_values = current_key_values,
                logits = logits,
                hidden_states = None,
                attentions = None,
            )


# coding=utf-8

# The current implementation is mainly to adapt the training framework of the Transformers toolkit, 
# and replace the original model implementation.
# TODO we will change to our SAM implementation in the future, which will be a more efficient tokenizer

from .base_tokenizer import BaseTokenizer
from transformers import AutoTokenizer as transformers_GPTjTokenizer

GPTjTokenizer = BaseTokenizer(transformers_GPTjTokenizer)
